@@ -34,6 +34,7 @@
 #include "qmljsviewercontext.h"
 
 #include <cplusplus/cppmodelmanagerbase.h>
+#include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/runextensions.h>
 
@@ -244,7 +245,7 @@ void ModelManagerInterface::addTaskInternal(QFuture<void> result, const QString
 void ModelManagerInterface::loadQmlTypeDescriptionsInternal(const QString &resourcePath)
 {
     const QDir typeFileDir(resourcePath + QLatin1String("/qml-type-descriptions"));
-    const QStringList qmlTypesExtensions = QStringList() << QLatin1String("*.qmltypes");
+    const QStringList qmlTypesExtensions = QStringList("*.qmltypes");
     QFileInfoList qmlTypesFiles = typeFileDir.entryInfoList(
                 qmlTypesExtensions,
                 QDir::Files,
@@ -351,7 +352,7 @@ QFuture<void> ModelManagerInterface::refreshSourceFiles(const QStringList &sourc
 void ModelManagerInterface::fileChangedOnDisk(const QString &path)
 {
     Utils::runAsync(&ModelManagerInterface::parse,
-                    workingCopyInternal(), QStringList() << path,
+                    workingCopyInternal(), QStringList(path),
                     this, Dialect(Dialect::AnyLanguage), true);
 }
 
@@ -439,9 +440,9 @@ void ModelManagerInterface::iterateQrcFiles(ProjectExplorer::Project *project,
     } else {
         pInfos = projectInfos();
         if (resources == ActiveQrcResources) // make the result predictable
-            qSort(pInfos.begin(), pInfos.end(), &pInfoLessThanActive);
+            Utils::sort(pInfos, &pInfoLessThanActive);
         else
-            qSort(pInfos.begin(), pInfos.end(), &pInfoLessThanAll);
+            Utils::sort(pInfos, &pInfoLessThanAll);
     }
 
     QSet<QString> pathsChecked;
@@ -565,8 +566,9 @@ void ModelManagerInterface::updateProjectInfo(const ProjectInfo &pinfo, ProjectE
     updateSourceFiles(newFiles, false);
 
     // update qrc cache
+    m_qrcContents = pinfo.resourceFileContents;
     foreach (const QString &newQrc, pinfo.allResourceFiles)
-        m_qrcCache.addPath(newQrc);
+        m_qrcCache.addPath(newQrc, m_qrcContents.value(newQrc));
     foreach (const QString &oldQrc, oldInfo.allResourceFiles)
         m_qrcCache.removePath(oldQrc);
 
@@ -577,7 +579,7 @@ void ModelManagerInterface::updateProjectInfo(const ProjectInfo &pinfo, ProjectE
            &majorVersion, &minorVersion, &patchVersion) != 3)
         majorVersion = minorVersion = patchVersion = -1;
 
-    if (majorVersion > 4 || (majorVersion == 4 && (minorVersion > 8 || (majorVersion == 8
+    if (majorVersion > 4 || (majorVersion == 4 && (minorVersion > 8 || (minorVersion == 8
                                                                        && patchVersion >= 5)))) {
         m_pluginDumper->loadBuiltinTypes(pinfo);
     }
@@ -655,7 +657,7 @@ void ModelManagerInterface::emitDocumentChangedOnDisk(Document::Ptr doc)
 
 void ModelManagerInterface::updateQrcFile(const QString &path)
 {
-    m_qrcCache.updatePath(path);
+    m_qrcCache.updatePath(path, m_qrcContents.value(path));
 }
 
 void ModelManagerInterface::updateDocument(Document::Ptr doc)
@@ -780,7 +782,8 @@ static bool findNewQmlLibraryInPath(const QString &path,
     }
 
     // found a new library!
-    qmldirFile.open(QFile::ReadOnly);
+    if (!qmldirFile.open(QFile::ReadOnly))
+        return false;
     QString qmldirData = QString::fromUtf8(qmldirFile.readAll());
 
     QmlDirParser qmldirParser;
@@ -988,7 +991,7 @@ struct ScanItem {
 void ModelManagerInterface::importScan(QFutureInterface<void> &future,
                               ModelManagerInterface::WorkingCopy workingCopy,
                               PathsAndLanguages paths, ModelManagerInterface *modelManager,
-                              bool emitDocChangedOnDisk, bool libOnly)
+                              bool emitDocChangedOnDisk, bool libOnly, bool forceRescan)
 {
     // paths we have scanned for files and added to the files list
     QSet<QString> scannedPaths;
@@ -1006,7 +1009,7 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
         for (int i = 0; i < paths.size(); ++i) {
             PathAndLanguage pAndL = paths.at(i);
             QString cPath = QDir::cleanPath(pAndL.path().toString());
-            if (modelManager->m_scannedPaths.contains(cPath))
+            if (!forceRescan && modelManager->m_scannedPaths.contains(cPath))
                 continue;
             pathsToScan.append(ScanItem(cPath, 0, pAndL.language()));
             modelManager->m_scannedPaths.insert(cPath);
@@ -1022,13 +1025,15 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
         ScanItem toScan = pathsToScan.last();
         pathsToScan.pop_back();
         int pathBudget = (1 << (maxScanDepth + 2 - toScan.depth));
-        if (!scannedPaths.contains(toScan.path)) {
+        if (forceRescan || !scannedPaths.contains(toScan.path)) {
             QStringList importedFiles;
-            if (!findNewQmlLibraryInPath(toScan.path, snapshot, modelManager, &importedFiles,
+            if (forceRescan ||
+                    (!findNewQmlLibraryInPath(toScan.path, snapshot, modelManager, &importedFiles,
                                          &scannedPaths, &newLibraries, true)
-                    && !libOnly && snapshot.documentsInDirectory(toScan.path).isEmpty())
+                    && !libOnly && snapshot.documentsInDirectory(toScan.path).isEmpty())) {
                 importedFiles += filesInDirectoryForLanguages(toScan.path,
                                                      toScan.language.companionLanguages());
+            }
             workDone += 1;
             future.setProgressValue(progressRange * workDone / totalWork);
             if (!importedFiles.isEmpty()) {
@@ -1099,7 +1104,7 @@ void ModelManagerInterface::maybeScan(const PathsAndLanguages &importPaths)
     if (pathToScan.length() > 1) {
         QFuture<void> result = Utils::runAsync(&ModelManagerInterface::importScan,
                                                workingCopyInternal(), pathToScan,
-                                               this, true, true);
+                                               this, true, true, false);
         cleanupFutures();
         m_futures.append(result);
 
@@ -1227,7 +1232,7 @@ void ModelManagerInterface::queueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr
     QPair<CPlusPlus::Document::Ptr, bool> prev = m_queuedCppDocuments.value(doc->fileName());
     if (prev.first && prev.second)
         prev.first->releaseSourceAndAST();
-    m_queuedCppDocuments.insert(doc->fileName(), qMakePair(doc, scan));
+    m_queuedCppDocuments.insert(doc->fileName(), {doc, scan});
     m_updateCppQmlTypesTimer->start();
 }
 
@@ -1290,16 +1295,16 @@ bool rescanExports(const QString &fileName, FindExportedCppTypes &finder,
     return hasNewInfo;
 }
 
-void ModelManagerInterface::updateCppQmlTypes(QFutureInterface<void> &interface,
+void ModelManagerInterface::updateCppQmlTypes(QFutureInterface<void> &futureInterface,
                                      ModelManagerInterface *qmlModelManager,
                                      CPlusPlus::Snapshot snapshot,
                                      QHash<QString, QPair<CPlusPlus::Document::Ptr, bool> > documents)
 {
-    interface.setProgressRange(0, documents.size());
-    interface.setProgressValue(0);
+    futureInterface.setProgressRange(0, documents.size());
+    futureInterface.setProgressValue(0);
 
     CppDataHash newData;
-    QHash<QString, QStringList> newDeclarations;
+    QHash<QString, QList<CPlusPlus::Document::Ptr> > newDeclarations;
     {
         QMutexLocker locker(&qmlModelManager->m_cppDataMutex);
         newData = qmlModelManager->m_cppDataHash;
@@ -1311,42 +1316,45 @@ void ModelManagerInterface::updateCppQmlTypes(QFutureInterface<void> &interface,
     bool hasNewInfo = false;
     typedef QPair<CPlusPlus::Document::Ptr, bool> DocScanPair;
     foreach (const DocScanPair &pair, documents) {
-        if (interface.isCanceled())
+        if (futureInterface.isCanceled())
             return;
-        interface.setProgressValue(interface.progressValue() + 1);
+        futureInterface.setProgressValue(futureInterface.progressValue() + 1);
 
         CPlusPlus::Document::Ptr doc = pair.first;
         const bool scan = pair.second;
         const QString fileName = doc->fileName();
         if (!scan) {
             hasNewInfo = newData.remove(fileName) > 0 || hasNewInfo;
-            foreach (const QString &file, newDeclarations[fileName]) {
-                CPlusPlus::Document::Ptr doc = snapshot.document(file);
-                if (doc.isNull())
-                    continue;
-                finder(doc);
-                hasNewInfo = rescanExports(file, finder, newData) || hasNewInfo;
+            foreach (const CPlusPlus::Document::Ptr &savedDoc, newDeclarations.value(fileName)) {
+                finder(savedDoc);
+                hasNewInfo = rescanExports(savedDoc->fileName(), finder, newData) || hasNewInfo;
             }
             continue;
         }
 
         for (auto it = newDeclarations.begin(), end = newDeclarations.end(); it != end;) {
-            if (it->removeOne(fileName)) {
-                doc->releaseSourceAndAST();
-                if (it->isEmpty()) {
-                    it = newDeclarations.erase(it);
-                    continue;
+            for (auto docIt = it->begin(), endDocIt = it->end(); docIt != endDocIt;) {
+                CPlusPlus::Document::Ptr &savedDoc = *docIt;
+                if (savedDoc->fileName() == fileName) {
+                    savedDoc->releaseSourceAndAST();
+                    it->erase(docIt);
+                    break;
+                } else {
+                    ++docIt;
                 }
             }
-            ++it;
+            if (it->isEmpty())
+                it = newDeclarations.erase(it);
+            else
+                ++it;
         }
 
         foreach (const QString &declarationFile, finder(doc)) {
-            newDeclarations[declarationFile].append(fileName);
+            newDeclarations[declarationFile].append(doc);
             doc->keepSourceAndAST(); // keep for later reparsing when dependent doc changes
         }
 
-        hasNewInfo = rescanExports(doc->fileName(), finder, newData) || hasNewInfo;
+        hasNewInfo = rescanExports(fileName, finder, newData) || hasNewInfo;
         doc->releaseSourceAndAST();
     }
 
