@@ -45,7 +45,6 @@
 #include <utils/hostosinfo.h>
 
 #include <QDirIterator>
-#include <QTcpServer>
 
 using namespace Debugger;
 using namespace ProjectExplorer;
@@ -95,42 +94,48 @@ static QString toNdkArch(const QString &arch)
     return QLatin1String("arch-") + arch;
 }
 
-RunControl *AndroidDebugSupport::createDebugRunControl(AndroidRunConfiguration *runConfig, QString *errorMessage)
+AndroidDebugSupport::AndroidDebugSupport(RunControl *runControl)
+    : Debugger::DebuggerRunTool(runControl)
 {
+    setDisplayName("AndroidDebugger");
+    m_runner = new AndroidRunner(runControl);
+    addStartDependency(m_runner);
+}
+
+void AndroidDebugSupport::start()
+{
+    auto runConfig = runControl()->runConfiguration();
     Target *target = runConfig->target();
+    Kit *kit = target->kit();
 
     DebuggerStartParameters params;
     params.startMode = AttachToRemoteServer;
     params.displayName = AndroidManager::packageName(target);
-    params.remoteSetupNeeded = true;
     params.useContinueInsteadOfRun = true;
+    params.attachPID = m_runner->pid();
     if (!Utils::HostOsInfo::isWindowsHost() &&
             AndroidConfigurations::currentConfig().ndkVersion() >= QVersionNumber(11, 0, 0)) {
         params.useTargetAsync = true;
     }
 
-    auto aspect = runConfig->extraAspect<DebuggerRunConfigurationAspect>();
-    if (aspect->useCppDebugger()) {
-        Kit *kit = target->kit();
-        params.symbolFile = target->activeBuildConfiguration()->buildDirectory().toString()
-                                     + QLatin1String("/app_process");
+    if (isCppDebugging()) {
+        Utils::Port gdbServerPort = m_runner->gdbServerPort();
+        params.symbolFile = target->activeBuildConfiguration()->buildDirectory().toString() + "/app_process";
         params.skipExecutableValidation = true;
-        params.remoteChannel = runConfig->remoteChannel();
+        params.useExtendedRemote = true;
+        params.remoteChannel = ":" + gdbServerPort.toString();
         params.solibSearchPath = AndroidManager::androidQtSupport(target)->soLibSearchPath(target);
         QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(kit);
         params.solibSearchPath.append(qtSoPaths(version));
         params.solibSearchPath.append(uniquePaths(AndroidManager::androidQtSupport(target)->androidExtraLibs(target)));
-        params.sysRoot = AndroidConfigurations::currentConfig().ndkLocation().appendPath(QLatin1String("platforms"))
-                                                     .appendPath(QLatin1String("android-") + QString::number(AndroidManager::minimumSDK(target)))
+        params.sysRoot = AndroidConfigurations::currentConfig().ndkLocation().appendPath("platforms")
+                                                     .appendPath(QString("android-%1").arg(AndroidManager::minimumSDK(target)))
                                                      .appendPath(toNdkArch(AndroidManager::targetArch(target))).toString();
     }
-    if (aspect->useQmlDebugger()) {
-        QTcpServer server;
-        QTC_ASSERT(server.listen(QHostAddress::LocalHost)
-                   || server.listen(QHostAddress::LocalHostIPv6), return 0);
-        params.qmlServer.host = server.serverAddress().toString();
+    if (isQmlDebugging()) {
+        params.qmlServer.host = m_runner->qmlServerHost();
+        params.qmlServer.port = m_runner->qmlServerPort();
         //TODO: Not sure if these are the right paths.
-        Kit *kit = target->kit();
         QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(kit);
         if (version) {
             const QString qmlQtDir = version->qmakeProperty("QT_INSTALL_QML");
@@ -138,68 +143,18 @@ RunControl *AndroidDebugSupport::createDebugRunControl(AndroidRunConfiguration *
         }
     }
 
-    DebuggerRunControl * const debuggerRunControl = createDebuggerRunControl(params, runConfig, errorMessage);
-    new AndroidDebugSupport(runConfig, debuggerRunControl);
-    return debuggerRunControl;
-}
-
-AndroidDebugSupport::AndroidDebugSupport(AndroidRunConfiguration *runConfig,
-    DebuggerRunControl *runControl)
-    : QObject(runControl),
-      m_runControl(runControl),
-      m_runner(new AndroidRunner(this, runConfig, runControl->runMode()))
-{
-    QTC_ASSERT(runControl, return);
-
-    connect(m_runControl, &RunControl::finished,
-            m_runner, &AndroidRunner::stop);
-
-    connect(m_runControl, &DebuggerRunControl::requestRemoteSetup,
-            m_runner, &AndroidRunner::start);
+    setStartParameters(params);
 
     // FIXME: Move signal to base class and generalize handling.
-    connect(m_runControl, &DebuggerRunControl::aboutToNotifyInferiorSetupOk,
+    connect(this, &DebuggerRunTool::aboutToNotifyInferiorSetupOk,
             m_runner, &AndroidRunner::remoteDebuggerRunning);
 
-    connect(m_runner, &AndroidRunner::remoteServerRunning,
-        [this](const QByteArray &serverChannel, int pid) {
-            QTC_ASSERT(m_runControl, return);
-            m_runControl->notifyEngineRemoteServerRunning(serverChannel, pid);
-        });
-
-    connect(m_runner, &AndroidRunner::remoteProcessStarted,
-            this, &AndroidDebugSupport::handleRemoteProcessStarted);
-
-    connect(m_runner, &AndroidRunner::remoteProcessFinished,
-        [this](const QString &errorMsg) {
-            QTC_ASSERT(m_runControl, return);
-            m_runControl->appendMessage(errorMsg, Utils::DebugFormat);
-            QMetaObject::invokeMethod(m_runControl, "notifyInferiorExited", Qt::QueuedConnection);
-        });
-
-    connect(m_runner, &AndroidRunner::remoteErrorOutput,
-        [this](const QString &output) {
-            QTC_ASSERT(m_runControl, return);
-            m_runControl->showMessage(output, AppError);
-        });
-
-    connect(m_runner, &AndroidRunner::remoteOutput,
-        [this](const QString &output) {
-            QTC_ASSERT(m_runControl, return);
-            m_runControl->showMessage(output, AppOutput);
-        });
+    DebuggerRunTool::start();
 }
 
-void AndroidDebugSupport::handleRemoteProcessStarted(Utils::Port gdbServerPort, Utils::Port qmlPort)
+void AndroidDebugSupport::stop()
 {
-    disconnect(m_runner, &AndroidRunner::remoteProcessStarted,
-               this, &AndroidDebugSupport::handleRemoteProcessStarted);
-    QTC_ASSERT(m_runControl, return);
-    RemoteSetupResult result;
-    result.success = true;
-    result.gdbServerPort = gdbServerPort;
-    result.qmlServerPort = qmlPort;
-    m_runControl->notifyEngineRemoteSetupFinished(result);
+    DebuggerRunTool::stop();
 }
 
 } // namespace Internal
