@@ -1,8 +1,8 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (C) 2015 BogDan Vatra <bog_dan_ro@yahoo.com>
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 BogDan Vatra <bog_dan_ro@yahoo.com>
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -10,22 +10,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -38,6 +33,7 @@
 #include "androidmanager.h"
 #include "androidconstants.h"
 #include "androidglobal.h"
+#include "androidavdmanager.h"
 
 #include <coreplugin/fileutils.h>
 #include <coreplugin/icore.h>
@@ -52,84 +48,54 @@
 
 #include <qtsupport/qtkitinformation.h>
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
+#include <utils/synchronousprocess.h>
 
 #include <QInputDialog>
 #include <QMessageBox>
 
 
+using namespace ProjectExplorer;
 using namespace Android;
 using namespace Android::Internal;
 
 const QLatin1String UninstallPreviousPackageKey("UninstallPreviousPackage");
 const QLatin1String InstallFailedInconsistentCertificatesString("INSTALL_PARSE_FAILED_INCONSISTENT_CERTIFICATES");
-const QLatin1String InstallFailedInconsistentCertificatesString2("INSTALL_FAILED_UPDATE_INCOMPATIBLE");
+const QLatin1String InstallFailedUpdateIncompatible("INSTALL_FAILED_UPDATE_INCOMPATIBLE");
+const QLatin1String InstallFailedPermissionModelDowngrade("INSTALL_FAILED_PERMISSION_MODEL_DOWNGRADE");
+const QLatin1String InstallFailedVersionDowngrade("INSTALL_FAILED_VERSION_DOWNGRADE");
 const Core::Id AndroidDeployQtStep::Id("Qt4ProjectManager.AndroidDeployQtStep");
 
 //////////////////
 // AndroidDeployQtStepFactory
 /////////////////
 
+
 AndroidDeployQtStepFactory::AndroidDeployQtStepFactory(QObject *parent)
     : IBuildStepFactory(parent)
 {
 }
 
-QList<Core::Id> AndroidDeployQtStepFactory::availableCreationIds(ProjectExplorer::BuildStepList *parent) const
+QList<BuildStepInfo> AndroidDeployQtStepFactory::availableSteps(BuildStepList *parent) const
 {
-    if (parent->id() != ProjectExplorer::Constants::BUILDSTEPS_DEPLOY)
-        return QList<Core::Id>();
-    if (!AndroidManager::supportsAndroid(parent->target()))
-        return QList<Core::Id>();
-    if (parent->contains(AndroidDeployQtStep::Id))
-        return QList<Core::Id>();
-    return QList<Core::Id>() << AndroidDeployQtStep::Id;
-}
+    if (parent->id() != ProjectExplorer::Constants::BUILDSTEPS_DEPLOY
+            || !AndroidManager::supportsAndroid(parent->target())
+            || parent->contains(AndroidDeployQtStep::Id))
+        return {};
 
-QString AndroidDeployQtStepFactory::displayNameForId(Core::Id id) const
-{
-    if (id == AndroidDeployQtStep::Id)
-        return tr("Deploy to Android device or emulator");
-    return QString();
-}
-
-bool AndroidDeployQtStepFactory::canCreate(ProjectExplorer::BuildStepList *parent, Core::Id id) const
-{
-    return availableCreationIds(parent).contains(id);
+    return {{AndroidDeployQtStep::Id, tr("Deploy to Android device or emulator")}};
 }
 
 ProjectExplorer::BuildStep *AndroidDeployQtStepFactory::create(ProjectExplorer::BuildStepList *parent, Core::Id id)
 {
-    Q_ASSERT(canCreate(parent, id));
     Q_UNUSED(id);
     return new AndroidDeployQtStep(parent);
 }
 
-bool AndroidDeployQtStepFactory::canRestore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map) const
-{
-    return canCreate(parent, ProjectExplorer::idFromMap(map));
-}
-
-ProjectExplorer::BuildStep *AndroidDeployQtStepFactory::restore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map)
-{
-    Q_ASSERT(canRestore(parent, map));
-    AndroidDeployQtStep * const step = new AndroidDeployQtStep(parent);
-    if (!step->fromMap(map)) {
-        delete step;
-        return 0;
-    }
-    return step;
-}
-
-bool AndroidDeployQtStepFactory::canClone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *product) const
-{
-    return canCreate(parent, product->id());
-}
-
 ProjectExplorer::BuildStep *AndroidDeployQtStepFactory::clone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *product)
 {
-    Q_ASSERT(canClone(parent, product));
     return new AndroidDeployQtStep(parent, static_cast<AndroidDeployQtStep *>(product));
 }
 
@@ -166,23 +132,31 @@ void AndroidDeployQtStep::ctor()
             this, &AndroidDeployQtStep::slotSetSerialNumber);
 }
 
-bool AndroidDeployQtStep::init()
+static AndroidDeviceInfo earlierDeviceInfo(QList<const ProjectExplorer::BuildStep *> &earlierSteps, Core::Id id)
 {
+    const ProjectExplorer::BuildStep *bs
+            = Utils::findOrDefault(earlierSteps, Utils::equal(&ProjectExplorer::BuildStep::id, id));
+    return bs ? static_cast<const AndroidDeployQtStep *>(bs)->deviceInfo() : AndroidDeviceInfo();
+}
+
+bool AndroidDeployQtStep::init(QList<const BuildStep *> &earlierSteps)
+{
+    Q_UNUSED(earlierSteps);
     m_androiddeployqtArgs.clear();
 
     if (AndroidManager::checkForQt51Files(project()->projectDirectory()))
-        emit addOutput(tr("Found old folder \"android\" in source directory. Qt 5.2 does not use that folder by default."), ErrorOutput);
+        emit addOutput(tr("Found old folder \"android\" in source directory. Qt 5.2 does not use that folder by default."), OutputFormat::Stderr);
 
     m_targetArch = AndroidManager::targetArch(target());
     if (m_targetArch.isEmpty()) {
-        emit addOutput(tr("No Android arch set by the .pro file."), ErrorOutput);
+        emit addOutput(tr("No Android arch set by the .pro file."), OutputFormat::Stderr);
         return false;
     }
 
     AndroidBuildApkStep *androidBuildApkStep
         = AndroidGlobal::buildStep<AndroidBuildApkStep>(target()->activeBuildConfiguration());
     if (!androidBuildApkStep) {
-        emit addOutput(tr("Cannot find the android build step."), ErrorOutput);
+        emit addOutput(tr("Cannot find the android build step."), OutputFormat::Stderr);
         return false;
     }
 
@@ -190,8 +164,13 @@ bool AndroidDeployQtStep::init()
     AndroidConfigurations::Options options = AndroidConfigurations::None;
     if (androidBuildApkStep->deployAction() == AndroidBuildApkStep::DebugDeployment)
         options = AndroidConfigurations::FilterAndroid5;
-    AndroidDeviceInfo info = AndroidConfigurations::showDeviceDialog(project(), deviceAPILevel, m_targetArch, options);
-    if (info.serialNumber.isEmpty() && info.avdname.isEmpty()) // aborted
+    AndroidDeviceInfo info = earlierDeviceInfo(earlierSteps, Id);
+    if (!info.isValid()) {
+        info = AndroidConfigurations::showDeviceDialog(project(), deviceAPILevel, m_targetArch, options);
+        m_deviceInfo = info; // Keep around for later steps
+    }
+
+    if (!info.isValid()) // aborted
         return false;
 
     m_avdName = info.avdname;
@@ -201,7 +180,7 @@ bool AndroidDeployQtStep::init()
     m_libdir = QLatin1String("lib");
     if (info.cpuAbi.contains(QLatin1String("arm64-v8a")) ||
             info.cpuAbi.contains(QLatin1String("x86_64"))) {
-        ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+        ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID);
         if (tc && tc->targetAbi().wordWidth() == 64) {
             m_appProcessBinaries << QLatin1String("/system/bin/app_process64");
             m_libdir += QLatin1String("64");
@@ -229,7 +208,7 @@ bool AndroidDeployQtStep::init()
     if (m_useAndroiddeployqt) {
         Utils::FileName tmp = AndroidManager::androidQtSupport(target())->androiddeployqtPath(target());
         if (tmp.isEmpty()) {
-            emit addOutput(tr("Cannot find the androiddeployqt tool."), ErrorOutput);
+            emit addOutput(tr("Cannot find the androiddeployqt tool."), OutputFormat::Stderr);
             return false;
         }
 
@@ -243,7 +222,7 @@ bool AndroidDeployQtStep::init()
         Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("--input"));
         tmp = AndroidManager::androidQtSupport(target())->androiddeployJsonPath(target());
         if (tmp.isEmpty()) {
-            emit addOutput(tr("Cannot find the androiddeploy Json file."), ErrorOutput);
+            emit addOutput(tr("Cannot find the androiddeploy Json file."), OutputFormat::Stderr);
             return false;
         }
         Utils::QtcProcess::addArg(&m_androiddeployqtArgs, tmp.toString());
@@ -284,14 +263,14 @@ bool AndroidDeployQtStep::init()
 
     m_adbPath = AndroidConfigurations::currentConfig().adbToolPath().toString();
 
-    if (AndroidConfigurations::currentConfig().findAvd(m_avdName).isEmpty())
-        AndroidConfigurations::currentConfig().startAVDAsync(m_avdName);
+    AndroidAvdManager avdManager;
+    if (avdManager.findAvd(m_avdName).isEmpty())
+        avdManager.startAvdAsync(m_avdName);
     return true;
 }
 
-AndroidDeployQtStep::DeployResult AndroidDeployQtStep::runDeploy(QFutureInterface<bool> &fi)
+AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::runDeploy(QFutureInterface<bool> &fi)
 {
-    m_installOk = true;
     QString args;
     if (m_useAndroiddeployqt) {
         args = m_androiddeployqtArgs;
@@ -308,11 +287,11 @@ AndroidDeployQtStep::DeployResult AndroidDeployQtStep::runDeploy(QFutureInterfac
         if (m_uninstallPreviousPackageRun) {
             const QString packageName = AndroidManager::packageName(m_manifestName);
             if (packageName.isEmpty()) {
-                emit addOutput(tr("Cannot find the package name."), ErrorOutput);
+                emit addOutput(tr("Cannot find the package name."), OutputFormat::Stderr);
                 return Failure;
             }
 
-            emit addOutput(tr("Uninstall previous package %1.").arg(packageName), MessageOutput);
+            emit addOutput(tr("Uninstall previous package %1.").arg(packageName), OutputFormat::NormalMessage);
             runCommand(m_adbPath,
                        AndroidDeviceInfo::adbSelector(m_serialNumber)
                        << QLatin1String("uninstall") << packageName);
@@ -334,18 +313,22 @@ AndroidDeployQtStep::DeployResult AndroidDeployQtStep::runDeploy(QFutureInterfac
     if (Utils::HostOsInfo::isWindowsHost())
         m_process->setUseCtrlCStub(true);
 
+    DeployErrorCode deployError = NoError;
     connect(m_process, &Utils::QtcProcess::readyReadStandardOutput,
-            this, &AndroidDeployQtStep::processReadyReadStdOutput, Qt::DirectConnection);
+            std::bind(&AndroidDeployQtStep::processReadyReadStdOutput, this, std::ref(deployError)));
     connect(m_process, &Utils::QtcProcess::readyReadStandardError,
-            this, &AndroidDeployQtStep::processReadyReadStdError, Qt::DirectConnection);
+            std::bind(&AndroidDeployQtStep::processReadyReadStdError, this, std::ref(deployError)));
 
     m_process->start();
 
     emit addOutput(tr("Starting: \"%1\" %2")
                    .arg(QDir::toNativeSeparators(m_command), args),
-                   BuildStep::MessageOutput);
+                   BuildStep::OutputFormat::NormalMessage);
 
     while (!m_process->waitForFinished(200)) {
+        if (m_process->state() == QProcess::NotRunning)
+            break;
+
         if (fi.isCanceled()) {
             m_process->kill();
             m_process->waitForFinished();
@@ -353,12 +336,16 @@ AndroidDeployQtStep::DeployResult AndroidDeployQtStep::runDeploy(QFutureInterfac
     }
 
     QString line = QString::fromLocal8Bit(m_process->readAllStandardError());
-    if (!line.isEmpty())
+    if (!line.isEmpty()) {
+        deployError |= parseDeployErrors(line);
         stdError(line);
+    }
 
     line = QString::fromLocal8Bit(m_process->readAllStandardOutput());
-    if (!line.isEmpty())
+    if (!line.isEmpty()) {
+        deployError |= parseDeployErrors(line);
         stdOutput(line);
+    }
 
     QProcess::ExitStatus exitStatus = m_process->exitStatus();
     int exitCode = m_process->exitCode();
@@ -367,33 +354,56 @@ AndroidDeployQtStep::DeployResult AndroidDeployQtStep::runDeploy(QFutureInterfac
 
     if (exitStatus == QProcess::NormalExit && exitCode == 0) {
         emit addOutput(tr("The process \"%1\" exited normally.").arg(m_command),
-                       BuildStep::MessageOutput);
+                       BuildStep::OutputFormat::NormalMessage);
     } else if (exitStatus == QProcess::NormalExit) {
         emit addOutput(tr("The process \"%1\" exited with code %2.")
                        .arg(m_command, QString::number(exitCode)),
-                       BuildStep::ErrorMessageOutput);
+                       BuildStep::OutputFormat::ErrorMessage);
     } else {
-        emit addOutput(tr("The process \"%1\" crashed.").arg(m_command), BuildStep::ErrorMessageOutput);
+        emit addOutput(tr("The process \"%1\" crashed.").arg(m_command), BuildStep::OutputFormat::ErrorMessage);
     }
 
     if (exitCode == 0 && exitStatus == QProcess::NormalExit) {
-        if (!m_installOk) {
-            if (!m_uninstallPreviousPackageRun)
-                return AskUinstall;
-            else
-                return Failure;
+        if (deployError != NoError && m_uninstallPreviousPackageRun) {
+            deployError = Failure;
         }
-        return Success;
+    } else {
+        deployError = Failure;
     }
-    return Failure;
+
+    return deployError;
 }
 
-void AndroidDeployQtStep::slotAskForUninstall()
+void AndroidDeployQtStep::slotAskForUninstall(DeployErrorCode errorCode)
 {
-    int button = QMessageBox::critical(0, tr("Install failed"),
-                                       tr("Another application with the same package id but signed with "
-                                          "different certificate already exists.\n"
-                                          "Do you want to uninstall the existing package?"),
+    Q_ASSERT(errorCode > 0);
+
+    QString uninstallMsg = tr("Deployment failed with the following errors:\n\n");
+    uint errorCodeFlags = errorCode;
+    uint mask = 1;
+    while (errorCodeFlags) {
+      switch (errorCodeFlags & mask) {
+      case DeployErrorCode::PermissionModelDowngrade:
+          uninstallMsg += InstallFailedPermissionModelDowngrade+"\n";
+          break;
+      case InconsistentCertificates:
+          uninstallMsg += InstallFailedInconsistentCertificatesString+"\n";
+          break;
+      case UpdateIncompatible:
+          uninstallMsg += InstallFailedUpdateIncompatible+"\n";
+          break;
+      case VersionDowngrade:
+          uninstallMsg += InstallFailedVersionDowngrade+"\n";
+          break;
+      default:
+          break;
+      }
+      errorCodeFlags &= ~mask;
+      mask <<= 1;
+    }
+
+    uninstallMsg.append(tr("\nUninstalling the installed package may solve the issue.\nDo you want to uninstall the existing package?"));
+    int button = QMessageBox::critical(0, tr("Install failed"), uninstallMsg,
                                        QMessageBox::Yes, QMessageBox::No);
     m_askForUinstall = button == QMessageBox::Yes;
 }
@@ -406,26 +416,25 @@ void AndroidDeployQtStep::slotSetSerialNumber(const QString &serialNumber)
 void AndroidDeployQtStep::run(QFutureInterface<bool> &fi)
 {
     if (!m_avdName.isEmpty()) {
-        QString serialNumber = AndroidConfigurations::currentConfig().waitForAvd(m_avdName, fi);
+        QString serialNumber = AndroidAvdManager().waitForAvd(m_avdName, fi);
         if (serialNumber.isEmpty()) {
-            fi.reportResult(false);
-            emit finished();
+            reportRunResult(fi, false);
             return;
         }
         m_serialNumber = serialNumber;
         emit setSerialNumber(serialNumber);
     }
 
-    DeployResult returnValue = runDeploy(fi);
-    if (returnValue == AskUinstall) {
-        emit askForUninstall();
+    DeployErrorCode returnValue = runDeploy(fi);
+    if (returnValue > DeployErrorCode::NoError && returnValue < DeployErrorCode::Failure) {
+        emit askForUninstall(returnValue);
         if (m_askForUinstall) {
             m_uninstallPreviousPackageRun = true;
             returnValue = runDeploy(fi);
         }
     }
 
-    emit addOutput(tr("Pulling files necessary for debugging."), MessageOutput);
+    emit addOutput(tr("Pulling files necessary for debugging."), OutputFormat::NormalMessage);
 
     QString localAppProcessFile = QString::fromLatin1("%1/app_process").arg(m_buildDirectory);
     QFile::remove(localAppProcessFile);
@@ -443,7 +452,7 @@ void AndroidDeployQtStep::run(QFutureInterface<bool> &fi)
         returnValue = Failure;
         emit addOutput(tr("Package deploy: Failed to pull \"%1\" to \"%2\".")
                        .arg(m_appProcessBinaries.join(QLatin1String("\", \"")))
-                       .arg(localAppProcessFile), ErrorMessageOutput);
+                       .arg(localAppProcessFile), OutputFormat::ErrorMessage);
     }
 
     runCommand(m_adbPath,
@@ -451,31 +460,22 @@ void AndroidDeployQtStep::run(QFutureInterface<bool> &fi)
                << QLatin1String("/system/") + m_libdir + QLatin1String("/libc.so")
                << QString::fromLatin1("%1/libc.so").arg(m_buildDirectory));
 
-    fi.reportResult(returnValue == Success ? true : false);
-    fi.reportFinished();
+    reportRunResult(fi, returnValue == NoError);
 }
 
 void AndroidDeployQtStep::runCommand(const QString &program, const QStringList &arguments)
 {
-    QProcess buildProc;
-    emit addOutput(tr("Package deploy: Running command \"%1 %2\".").arg(program).arg(arguments.join(QLatin1Char(' '))), BuildStep::MessageOutput);
-    buildProc.start(program, arguments);
-    if (!buildProc.waitForStarted()) {
-        emit addOutput(tr("Packaging error: Could not start command \"%1 %2\". Reason: %3")
-            .arg(program).arg(arguments.join(QLatin1Char(' '))).arg(buildProc.errorString()), BuildStep::ErrorMessageOutput);
-        return;
-    }
-    if (!buildProc.waitForFinished(2 * 60 * 1000)
-            || buildProc.error() != QProcess::UnknownError
-            || buildProc.exitCode() != 0) {
-        QString mainMessage = tr("Packaging error: Command \"%1 %2\" failed.")
-                .arg(program).arg(arguments.join(QLatin1Char(' ')));
-        if (buildProc.error() != QProcess::UnknownError)
-            mainMessage += QLatin1Char(' ') + tr("Reason: %1").arg(buildProc.errorString());
-        else
-            mainMessage += tr("Exit code: %1").arg(buildProc.exitCode());
-        emit addOutput(mainMessage, BuildStep::ErrorMessageOutput);
-    }
+    Utils::SynchronousProcess buildProc;
+    buildProc.setTimeoutS(2 * 60);
+    emit addOutput(tr("Package deploy: Running command \"%1 %2\".").arg(program).arg(arguments.join(QLatin1Char(' '))), BuildStep::OutputFormat::NormalMessage);
+    Utils::SynchronousProcessResponse response = buildProc.run(program, arguments);
+    if (response.result != Utils::SynchronousProcessResponse::Finished || response.exitCode != 0)
+        emit addOutput(response.exitMessage(program, 2 * 60), BuildStep::OutputFormat::ErrorMessage);
+}
+
+AndroidDeviceInfo AndroidDeployQtStep::deviceInfo() const
+{
+    return m_deviceInfo;
 }
 
 ProjectExplorer::BuildStepConfigWidget *AndroidDeployQtStep::createConfigWidget()
@@ -483,38 +483,50 @@ ProjectExplorer::BuildStepConfigWidget *AndroidDeployQtStep::createConfigWidget(
     return new AndroidDeployQtWidget(this);
 }
 
-void AndroidDeployQtStep::processReadyReadStdOutput()
+void AndroidDeployQtStep::processReadyReadStdOutput(DeployErrorCode &errorCode)
 {
     m_process->setReadChannel(QProcess::StandardOutput);
     while (m_process->canReadLine()) {
         QString line = QString::fromLocal8Bit(m_process->readLine());
+        errorCode |= parseDeployErrors(line);
         stdOutput(line);
     }
 }
 
 void AndroidDeployQtStep::stdOutput(const QString &line)
 {
-    if (line.contains(InstallFailedInconsistentCertificatesString)
-            || line.contains(InstallFailedInconsistentCertificatesString2))
-        m_installOk = false;
-    emit addOutput(line, BuildStep::NormalOutput, BuildStep::DontAppendNewline);
+    emit addOutput(line, BuildStep::OutputFormat::Stdout, BuildStep::DontAppendNewline);
 }
 
-void AndroidDeployQtStep::processReadyReadStdError()
+void AndroidDeployQtStep::processReadyReadStdError(DeployErrorCode &errorCode)
 {
     m_process->setReadChannel(QProcess::StandardError);
     while (m_process->canReadLine()) {
         QString line = QString::fromLocal8Bit(m_process->readLine());
+        errorCode |= parseDeployErrors(line);
         stdError(line);
     }
 }
 
 void AndroidDeployQtStep::stdError(const QString &line)
 {
-    if (line.contains(InstallFailedInconsistentCertificatesString)
-            || line.contains(InstallFailedInconsistentCertificatesString2))
-        m_installOk = false;
-    emit addOutput(line, BuildStep::ErrorOutput, BuildStep::DontAppendNewline);
+    emit addOutput(line, BuildStep::OutputFormat::Stderr, BuildStep::DontAppendNewline);
+}
+
+AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::parseDeployErrors(QString &deployOutputLine) const
+{
+    DeployErrorCode errorCode = NoError;
+
+    if (deployOutputLine.contains(InstallFailedInconsistentCertificatesString))
+        errorCode |= InconsistentCertificates;
+    if (deployOutputLine.contains(InstallFailedUpdateIncompatible))
+        errorCode |= UpdateIncompatible;
+    if (deployOutputLine.contains(InstallFailedPermissionModelDowngrade))
+        errorCode |= PermissionModelDowngrade;
+    if (deployOutputLine.contains(InstallFailedVersionDowngrade))
+        errorCode |= VersionDowngrade;
+
+    return errorCode;
 }
 
 bool AndroidDeployQtStep::fromMap(const QVariantMap &map)
