@@ -25,6 +25,7 @@
 
 #include "clangcodecompletion_test.h"
 
+#include "clangautomationutils.h"
 #include "../clangbackendipcintegration.h"
 #include "../clangcompletionassistinterface.h"
 #include "../clangmodelmanagersupport.h"
@@ -38,23 +39,13 @@
 #include <cpptools/cpptoolstestcase.h>
 #include <cpptools/modelmanagertesthelper.h>
 #include <cpptools/projectinfo.h>
-#include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/assistproposalitem.h>
-#include <texteditor/codeassist/completionassistprovider.h>
 #include <texteditor/codeassist/genericproposalmodel.h>
-#include <texteditor/codeassist/iassistprocessor.h>
-#include <texteditor/codeassist/iassistproposal.h>
 #include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 
-#include <clangbackendipc/cmbcompletecodemessage.h>
-#include <clangbackendipc/cmbendmessage.h>
-#include <clangbackendipc/cmbregisterprojectsforeditormessage.h>
-#include <clangbackendipc/cmbregistertranslationunitsforeditormessage.h>
-#include <clangbackendipc/cmbunregisterprojectsforeditormessage.h>
-#include <clangbackendipc/cmbunregistertranslationunitsforeditormessage.h>
-#include <clangbackendipc/registerunsavedfilesforeditormessage.h>
-#include <clangbackendipc/updatetranslationunitsforeditormessage.h>
+#include <clangbackendipc/clangcodemodelservermessages.h>
+
 #include <utils/changeset.h>
 #include <utils/qtcassert.h>
 
@@ -180,53 +171,6 @@ void insertTextAtTopOfEditor(TextEditor::BaseTextEditor *editor, const QByteArra
     cs.apply(&textCursor);
 }
 
-class WaitForAsyncCompletions
-{
-public:
-    enum WaitResult { GotResults, GotInvalidResults, Timeout };
-    WaitResult wait(TextEditor::IAssistProcessor *processor,
-                    TextEditor::AssistInterface *assistInterface);
-
-    TextEditor::IAssistProposalModel *proposalModel;
-};
-
-WaitForAsyncCompletions::WaitResult WaitForAsyncCompletions::wait(
-        TextEditor::IAssistProcessor *processor,
-        TextEditor::AssistInterface *assistInterface)
-{
-    QTC_ASSERT(processor, return Timeout);
-    QTC_ASSERT(assistInterface, return Timeout);
-
-    bool gotResults = false;
-
-    processor->setAsyncCompletionAvailableHandler(
-                [this, &gotResults] (TextEditor::IAssistProposal *proposal) {
-        QTC_ASSERT(proposal, return);
-        proposalModel = proposal->model();
-        delete proposal;
-        gotResults = true;
-    });
-
-    // Are there any immediate results?
-    if (TextEditor::IAssistProposal *proposal = processor->perform(assistInterface)) {
-        delete processor;
-        proposalModel = proposal->model();
-        delete proposal;
-        QTC_ASSERT(proposalModel, return GotInvalidResults);
-        return GotResults;
-    }
-
-    // There are not any, so wait for async results.
-    QElapsedTimer timer; timer.start();
-    while (!gotResults) {
-        if (timer.elapsed() >= 5 * 1000)
-            return Timeout;
-        QCoreApplication::processEvents();
-    }
-
-    return proposalModel ? GotResults : GotInvalidResults;
-}
-
 class ChangeDocumentReloadSetting
 {
 public:
@@ -250,13 +194,13 @@ class ChangeIpcSender
 public:
     ChangeIpcSender(IpcSenderInterface *ipcSender)
     {
-        auto &ipc = ModelManagerSupportClang::instance_forTestsOnly()->ipcCommunicator();
+        auto &ipc = ModelManagerSupportClang::instance()->ipcCommunicator();
         m_previousSender = ipc.setIpcSender(ipcSender);
     }
 
     ~ChangeIpcSender()
     {
-        auto &ipc = ModelManagerSupportClang::instance_forTestsOnly()->ipcCommunicator();
+        auto &ipc = ModelManagerSupportClang::instance()->ipcCommunicator();
         ipc.setIpcSender(m_previousSender);
     }
 
@@ -370,14 +314,14 @@ QString toString(const CompleteCodeMessage &)
     return QLatin1String("CompleteCodeMessage\n");
 }
 
-QString toString(const RequestDiagnosticsMessage &)
+QString toString(const RequestDocumentAnnotationsMessage &)
 {
-    return QStringLiteral("RequestDiagnosticsMessage\n");
+    return QStringLiteral("RequestDocumentAnnotationsMessage\n");
 }
 
-QString toString(const RequestHighlightingMessage &)
+QString toString(const RequestReferencesMessage &)
 {
-    return QStringLiteral("RequestHighlightingMessage\n");
+    return QStringLiteral("RequestReferencesMessage\n");
 }
 
 QString toString(const UpdateVisibleTranslationUnitsMessage &)
@@ -415,10 +359,10 @@ public:
     void completeCode(const CompleteCodeMessage &message) override
     { senderLog.append(toString(message)); }
 
-    void requestDiagnostics(const RequestDiagnosticsMessage &message) override
+    void requestDocumentAnnotations(const RequestDocumentAnnotationsMessage &message) override
     { senderLog.append(toString(message)); }
 
-    void requestHighlighting(const RequestHighlightingMessage &message) override
+    void requestReferences(const RequestReferencesMessage &message) override
     { senderLog.append(toString(message)); }
 
     void updateVisibleTranslationUnits(const UpdateVisibleTranslationUnitsMessage &message) override
@@ -428,51 +372,6 @@ public:
     QString senderLog;
 };
 
-const CppTools::ProjectPartHeaderPaths toHeaderPaths(const QStringList &paths)
-{
-    using namespace CppTools;
-
-    ProjectPartHeaderPaths result;
-    foreach (const QString &path, paths)
-        result << ProjectPartHeaderPath(path, ProjectPartHeaderPath::IncludePath);
-    return result;
-}
-
-using ProposalModel = QSharedPointer<TextEditor::IAssistProposalModel>;
-
-ProposalModel completionResults(
-        TextEditor::BaseTextEditor *textEditor,
-        const QStringList &includePaths = QStringList())
-{
-    using namespace TextEditor;
-
-    TextEditorWidget *textEditorWidget = qobject_cast<TextEditorWidget *>(textEditor->widget());
-    QTC_ASSERT(textEditorWidget, return ProposalModel());
-    AssistInterface *assistInterface = textEditorWidget->createAssistInterface(
-                TextEditor::Completion, TextEditor::ExplicitlyInvoked);
-    QTC_ASSERT(assistInterface, return ProposalModel());
-    if (!includePaths.isEmpty()) {
-        auto clangAssistInterface = static_cast<ClangCompletionAssistInterface *>(assistInterface);
-        clangAssistInterface->setHeaderPaths(toHeaderPaths(includePaths));
-    }
-
-    CompletionAssistProvider *assistProvider
-            = textEditor->textDocument()->completionAssistProvider();
-    QTC_ASSERT(qobject_cast<ClangCompletionAssistProvider *>(assistProvider),
-               return ProposalModel());
-    QTC_ASSERT(assistProvider, return ProposalModel());
-    QTC_ASSERT(assistProvider->runType() == IAssistProvider::Asynchronous, return ProposalModel());
-
-    IAssistProcessor *processor = assistProvider->createProcessor();
-    QTC_ASSERT(processor, return ProposalModel());
-
-    WaitForAsyncCompletions waitForCompletions;
-    const WaitForAsyncCompletions::WaitResult result = waitForCompletions.wait(processor,
-                                                                               assistInterface);
-    QTC_ASSERT(result == WaitForAsyncCompletions::GotResults, return ProposalModel());
-    return QSharedPointer<TextEditor::IAssistProposalModel>(waitForCompletions.proposalModel);
-}
-
 class TestDocument
 {
 public:
@@ -613,7 +512,6 @@ CppTools::ProjectPart::Ptr createProjectPart(const QStringList &files,
     projectPart->projectFile = QLatin1String("myproject.project");
     foreach (const QString &file, files)
         projectPart->files.append(ProjectFile(file, ProjectFile::classify(file)));
-    projectPart->languageVersion = ProjectPart::CXX11;
     projectPart->qtVersion = ProjectPart::NoQt;
     projectPart->projectDefines = defines.toUtf8();
 
@@ -630,7 +528,6 @@ CppTools::ProjectInfo createProjectInfo(ProjectExplorer::Project *project,
     const CppTools::ProjectPart::Ptr projectPart = createProjectPart(files, defines);
     ProjectInfo projectInfo = ProjectInfo(project);
     projectInfo.appendProjectPart(projectPart);
-    projectInfo.finish();
     return projectInfo;
 }
 
@@ -698,7 +595,7 @@ public:
         if (!textToInsert.isEmpty())
             openEditor.editor()->insert(textToInsert);
 
-        proposal = completionResults(openEditor.editor(), includePaths);
+        proposal = completionResults(openEditor.editor(), includePaths, 15000);
     }
 
     ProposalModel proposal;
@@ -894,7 +791,7 @@ void ClangCodeCompletionTest::testCompleteGlobals()
     ProjectLessCompletionTest t("globalCompletion.cpp");
 
     QVERIFY(hasItem(t.proposal, "globalVariable", "int globalVariable"));
-    QVERIFY(hasItem(t.proposal, "globalFunction", "void globalFunction ()"));
+    QVERIFY(hasItem(t.proposal, "globalFunction", "void globalFunction()"));
     QVERIFY(hasItem(t.proposal, "GlobalClass", "GlobalClass"));
     QVERIFY(hasItem(t.proposal, "class", "class"));    // Keyword
     QVERIFY(hasSnippet(t.proposal, "class")); // Snippet
@@ -923,13 +820,14 @@ void ClangCodeCompletionTest::testCompleteFunctions()
     QVERIFY(hasItem(t.proposal, "TType&lt;QString&gt; f(bool)"));
 }
 
-void ClangCodeCompletionTest::testCompleteConstructorAndFallbackToGlobalCompletion()
+void ClangCodeCompletionTest::testCompleteConstructor()
 {
     ProjectLessCompletionTest t("constructorCompletion.cpp");
 
-    QVERIFY(hasItem(t.proposal, "globalVariable"));
-    QVERIFY(hasItem(t.proposal, "GlobalClassWithCustomConstructor"));
-    QVERIFY(!hasSnippet(t.proposal, "class"));
+    QVERIFY(!hasItem(t.proposal, "globalVariable"));
+    QVERIFY(!hasItem(t.proposal, "class"));
+    QVERIFY(hasItem(t.proposal, "Foo(int)"));
+    QVERIFY(hasItem(t.proposal, "Foo(int, double)"));
 }
 
 // Explicitly Inserting The Dot
@@ -1217,7 +1115,7 @@ void ClangCodeCompletionTest::testUpdateBackendAfterRestart()
     spy.senderLog.clear();
 
     // Kill backend process...
-    auto &ipcCommunicator = ModelManagerSupportClang::instance_forTestsOnly()->ipcCommunicator();
+    auto &ipcCommunicator = ModelManagerSupportClang::instance()->ipcCommunicator();
     ipcCommunicator.killBackendProcess();
     QSignalSpy waitForReinitializedBackend(&ipcCommunicator,
                                            SIGNAL(backendReinitialized()));
