@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef CPPEDITOR_H
-#define CPPEDITOR_H
+#pragma once
 
 #include "cppfunctiondecldeflink.h"
 
@@ -34,7 +33,9 @@
 
 namespace CppTools {
 class CppEditorOutline;
+class RefactoringEngineInterface;
 class SemanticInfo;
+class ProjectPart;
 }
 
 namespace CppEditor {
@@ -52,6 +53,9 @@ class CppEditor : public TextEditor::BaseTextEditor
 
 public:
     CppEditor();
+
+    static void decorateEditor(TextEditor::TextEditorWidget *editor);
+
 };
 
 class CppEditorWidget : public TextEditor::TextEditorWidget
@@ -80,7 +84,6 @@ public:
 
     void encourageApply() override;
 
-public slots:
     void paste() override;
     void cut() override;
     void selectAll() override;
@@ -95,6 +98,9 @@ public slots:
     bool selectBlockUp() override;
     bool selectBlockDown() override;
 
+    static void updateWidgetHighlighting(QWidget *widget, bool highlight);
+    static bool isWidgetHighlighted(QWidget *widget);
+
 protected:
     bool event(QEvent *e) override;
     void contextMenuEvent(QContextMenuEvent *) override;
@@ -106,10 +112,9 @@ protected:
 
     void onRefactorMarkerClicked(const TextEditor::RefactorMarker &marker) override;
 
-protected slots:
     void slotCodeStyleSettingsChanged(const QVariant &) override;
 
-private slots:
+private:
     void updateFunctionDeclDefLink();
     void updateFunctionDeclDefLinkNow();
     void abortDeclDefLink();
@@ -123,15 +128,14 @@ private slots:
     void onIfdefedOutBlocksUpdated(unsigned revision,
                                    const QList<TextEditor::BlockRange> ifdefedOutBlocks);
 
+    void onShowInfoBarAction(const Core::Id &id, bool show);
+
     void updateSemanticInfo(const CppTools::SemanticInfo &semanticInfo,
                             bool updateUseSelectionSynchronously = false);
     void updatePreprocessorButtonTooltip();
 
-    void performQuickFix(int index);
-
     void processKeyNormally(QKeyEvent *e);
 
-private:
     void finalizeInitialization() override;
     void finalizeInitializationAfterDuplication(TextEditorWidget *other) override;
 
@@ -139,11 +143,16 @@ private:
 
     TextEditor::RefactorMarkers refactorMarkersWithoutClangMarkers() const;
 
+    CppTools::RefactoringEngineInterface *refactoringEngine() const;
+
+    void renameSymbolUnderCursorClang();
+    void renameSymbolUnderCursorBuiltin();
+
+    CppTools::ProjectPart *projectPart() const;
+
 private:
     QScopedPointer<CppEditorWidgetPrivate> d;
 };
 
 } // namespace Internal
 } // namespace CppEditor
-
-#endif // CPPEDITOR_H
