@@ -40,8 +40,8 @@ namespace Internal {
 
 #define CB(callback) [this](const DebuggerResponse &r) { callback(r); }
 
-GdbPlainEngine::GdbPlainEngine(const DebuggerRunParameters &startParameters)
-    : GdbEngine(startParameters)
+GdbPlainEngine::GdbPlainEngine(bool useTerminal)
+    : GdbEngine(useTerminal)
 {
     // Output
     connect(&m_outputCollector, &OutputCollector::byteDelivery,
@@ -52,14 +52,17 @@ void GdbPlainEngine::setupInferior()
 {
     QTC_ASSERT(state() == InferiorSetupRequested, qDebug() << state());
     setEnvironmentVariables();
-    if (!runParameters().inferior.commandLineArguments.isEmpty()) {
-        QString args = runParameters().inferior.commandLineArguments;
-        runCommand({"-exec-arguments " + args, NoFlags});
+    const DebuggerRunParameters &rp = runParameters();
+    if (!rp.inferior.workingDirectory.isEmpty())
+        runCommand({"cd " + rp.inferior.workingDirectory});
+    if (!rp.inferior.commandLineArguments.isEmpty()) {
+        QString args = rp.inferior.commandLineArguments;
+        runCommand({"-exec-arguments " + args});
     }
 
     QString executable = QFileInfo(runParameters().inferior.executable).absoluteFilePath();
-    runCommand({"-file-exec-and-symbols \"" + executable + '"', NoFlags,
-        CB(handleFileExecAndSymbols)});
+    runCommand({"-file-exec-and-symbols \"" + executable + '"',
+                CB(handleFileExecAndSymbols)});
 }
 
 void GdbPlainEngine::handleFileExecAndSymbols(const DebuggerResponse &response)
@@ -79,9 +82,9 @@ void GdbPlainEngine::handleFileExecAndSymbols(const DebuggerResponse &response)
 void GdbPlainEngine::runEngine()
 {
     if (runParameters().useContinueInsteadOfRun)
-        runCommand({"-exec-continue", RunRequest, CB(handleExecuteContinue)});
+        runCommand({"-exec-continue", DebuggerCommand::RunRequest, CB(handleExecuteContinue)});
     else
-        runCommand({"-exec-run", RunRequest, CB(handleExecRun)});
+        runCommand({"-exec-run", DebuggerCommand::RunRequest, CB(handleExecRun)});
 }
 
 void GdbPlainEngine::handleExecRun(const DebuggerResponse &response)
@@ -94,7 +97,7 @@ void GdbPlainEngine::handleExecRun(const DebuggerResponse &response)
         showMessage(msgInferiorSetupOk(), StatusBar);
         // FIXME: That's the wrong place for it.
         if (boolSetting(EnableReverseDebugging))
-            runCommand({"target record", NoFlags});
+            runCommand({"target record"});
     } else {
         QString msg = response.data["msg"].data();
         //QTC_CHECK(status() == InferiorRunOk);
@@ -121,10 +124,6 @@ void GdbPlainEngine::setupEngine()
     }
     gdbArgs.append("--tty=" + m_outputCollector.serverName());
 
-    QString workingDirectory = runParameters().inferior.workingDirectory;
-    if (!workingDirectory.isEmpty() && QFileInfo::exists(workingDirectory))
-        m_gdbProc.setWorkingDirectory(workingDirectory);
-
     startGdb(gdbArgs);
 }
 
