@@ -28,12 +28,13 @@
 #include <debugger/debuggeractions.h>
 #include <debugger/debuggercore.h>
 #include <debugger/debuggerprotocol.h>
-#include <debugger/debuggerstartparameters.h>
+#include <debugger/debuggerruncontrol.h>
 #include <debugger/procinterrupt.h>
 
 #include <coreplugin/messagebox.h>
 
 #include <utils/hostosinfo.h>
+#include <utils/qtcfallthrough.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 
@@ -54,12 +55,9 @@ namespace Internal {
 //
 ///////////////////////////////////////////////////////////////////////
 
-GdbRemoteServerEngine::GdbRemoteServerEngine(const DebuggerRunParameters &runParameters)
-    : GdbEngine(runParameters), m_startAttempted(false)
+GdbRemoteServerEngine::GdbRemoteServerEngine(bool useTerminal)
+    : GdbEngine(useTerminal)
 {
-    if (HostOsInfo::isWindowsHost())
-        m_gdbProc.setUseCtrlCStub(runParameters.useCtrlCStub); // This is only set for QNX
-
     connect(&m_uploadProc, &QProcess::errorOccurred, this, &GdbRemoteServerEngine::uploadProcError);
     connect(&m_uploadProc, &QProcess::readyReadStandardOutput,
             this, &GdbRemoteServerEngine::readUploadStandardOutput);
@@ -71,6 +69,9 @@ GdbRemoteServerEngine::GdbRemoteServerEngine(const DebuggerRunParameters &runPar
 
 void GdbRemoteServerEngine::setupEngine()
 {
+    if (HostOsInfo::isWindowsHost())
+        m_gdbProc.setUseCtrlCStub(runParameters().useCtrlCStub); // This is only set for QNX
+
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
     showMessage("TRYING TO START ADAPTER");
     QString serverStartScript = runParameters().serverStartScript;
@@ -84,14 +85,10 @@ void GdbRemoteServerEngine::setupEngine()
 
         m_uploadProc.start(arglist);
         m_uploadProc.waitForStarted();
+        m_uploadProc.waitForFinished();
     }
 
-    if (runParameters().remoteSetupNeeded) {
-        notifyEngineRequestRemoteSetup();
-    } else {
-        m_startAttempted = true;
-        startGdb();
-    }
+    startGdb();
 }
 
 void GdbRemoteServerEngine::uploadProcError(QProcess::ProcessError error)
@@ -147,13 +144,9 @@ void GdbRemoteServerEngine::readUploadStandardError()
 void GdbRemoteServerEngine::uploadProcFinished()
 {
     if (m_uploadProc.exitStatus() == QProcess::NormalExit && m_uploadProc.exitCode() == 0) {
-        if (!m_startAttempted)
-            startGdb();
+        // all good.
     } else {
-        RemoteSetupResult result;
-        result.success = false;
-        result.reason = m_uploadProc.errorString();
-        notifyEngineRemoteSetupFinished(result);
+        runTool()->reportFailure(tr("Upload failed: %1").arg(m_uploadProc.errorString()));
     }
 }
 
@@ -170,8 +163,7 @@ void GdbRemoteServerEngine::setupInferior()
 
     //const QByteArray sysroot = sp.sysroot.toLocal8Bit();
     //const QByteArray remoteArch = sp.remoteArchitecture.toLatin1();
-    const QString args = isMasterEngine() ? runParameters().inferior.commandLineArguments
-                          : masterEngine()->runParameters().inferior.commandLineArguments;
+    const QString args = runParameters().inferior.commandLineArguments;
 
 //    if (!remoteArch.isEmpty())
 //        postCommand("set architecture " + remoteArch);
@@ -376,7 +368,7 @@ void GdbRemoteServerEngine::handleAttach(const DebuggerResponse &response)
             notifyInferiorSetupFailed(msgPtraceError(runParameters().startMode));
             break;
         }
-        // if msg != "ptrace: ..." fall through
+        Q_FALLTHROUGH(); // if msg != "ptrace: ..."
     default:
         notifyInferiorSetupFailed(response.data["msg"].data());
     }
@@ -460,30 +452,5 @@ void GdbRemoteServerEngine::shutdownEngine()
     notifyAdapterShutdownOk();
 }
 
-void GdbRemoteServerEngine::notifyEngineRemoteServerRunning
-    (const QString &serverChannel, int inferiorPid)
-{
-    // Currently only used by Android support.
-    runParameters().attachPID = Utils::ProcessHandle(inferiorPid);
-    runParameters().remoteChannel = serverChannel;
-    runParameters().useExtendedRemote = true;
-    showMessage("NOTE: REMOTE SERVER RUNNING IN MULTIMODE");
-    m_startAttempted = true;
-    startGdb();
-}
-
-void GdbRemoteServerEngine::notifyEngineRemoteSetupFinished(const RemoteSetupResult &result)
-{
-    QTC_ASSERT(state() == EngineSetupRequested, qDebug() << state());
-    GdbEngine::notifyEngineRemoteSetupFinished(result);
-
-    if (result.success) {
-        if (!m_startAttempted)
-            startGdb();
-    } else {
-        handleAdapterStartFailed(result.reason);
-    }
-}
-
 } // namespace Internal
 } // namespace Debugger
