@@ -302,7 +302,7 @@ void GerritPlugin::updateActions(const VcsBase::VcsBasePluginState &state)
     const bool hasTopLevel = state.hasTopLevel();
     m_gerritCommand->action()->setEnabled(hasTopLevel);
     m_pushToGerritCommand->action()->setEnabled(hasTopLevel);
-    if (m_dialog)
+    if (m_dialog && m_dialog->isVisible())
         m_dialog->setCurrentPath(state.topLevel());
 }
 
@@ -315,7 +315,7 @@ void GerritPlugin::addToLocator(CommandLocator *locator)
 void GerritPlugin::push(const QString &topLevel)
 {
     // QScopedPointer is required to delete the dialog when leaving the function
-    GerritPushDialog dialog(topLevel, m_reviewers, ICore::mainWindow());
+    GerritPushDialog dialog(topLevel, m_reviewers, m_parameters, ICore::mainWindow());
 
     if (!dialog.isValid()) {
         QMessageBox::warning(ICore::mainWindow(), tr("Initialization Failed"),
@@ -371,6 +371,8 @@ void GerritPlugin::openView()
         connect(this, &GerritPlugin::fetchStarted, gd, &GerritDialog::fetchStarted);
         connect(this, &GerritPlugin::fetchFinished, gd, &GerritDialog::fetchFinished);
         m_dialog = gd;
+    } else {
+        m_dialog->refresh();
     }
     const Qt::WindowStates state = m_dialog->windowState();
     if (state & Qt::WindowMinimized)
