@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,26 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
-#ifndef IOSRUNCONFIGURATION_H
-#define IOSRUNCONFIGURATION_H
+
+#pragma once
 
 #include "iosconstants.h"
 #include "iosconfigurations.h"
@@ -37,9 +32,7 @@
 #include <projectexplorer/runconfiguration.h>
 #include <utils/fileutils.h>
 
-namespace QmakeProjectManager {
-class QmakeProFileNode;
-}
+namespace QmakeProjectManager { class QmakeProFile; }
 
 namespace Ios {
 namespace Internal {
@@ -60,35 +53,36 @@ public:
     Utils::OutputFormatter *createOutputFormatter() const override;
     IosDeployStep *deployStep() const;
 
-    QStringList commandLineArguments();
+    QString commandLineArguments() const;
     Utils::FileName profilePath() const;
     QString applicationName() const;
     Utils::FileName bundleDirectory() const;
     Utils::FileName localExecutable() const;
-    bool isEnabled() const;
-    QString disabledReason() const;
+    bool isEnabled() const override;
+    QString disabledReason() const override;
     IosDeviceType deviceType() const;
     void setDeviceType(const IosDeviceType &deviceType);
 
     bool fromMap(const QVariantMap &map) override;
     QVariantMap toMap() const override;
 
+    QString buildSystemTarget() const final;
+
 protected:
     IosRunConfiguration(ProjectExplorer::Target *parent, IosRunConfiguration *source);
 
-private slots:
-    void proFileUpdated(QmakeProjectManager::QmakeProFileNode *pro, bool success, bool parseInProgress);
-    void deviceChanges();
 signals:
     void localExecutableChanged();
+
 private:
+    void proFileUpdated(QmakeProjectManager::QmakeProFile *pro, bool success, bool parseInProgress);
+    void deviceChanges();
     void init();
     void enabledCheck();
     friend class IosRunConfigurationWidget;
     void updateDisplayNames();
 
     Utils::FileName m_profilePath;
-    QStringList m_arguments;
     QString m_lastDisabledReason;
     bool m_lastIsEnabled;
     bool m_parseInProgress;
@@ -98,5 +92,3 @@ private:
 
 } // namespace Internal
 } // namespace Ios
-
-#endif // IOSRUNCONFIGURATION_H
