@@ -43,6 +43,11 @@ CurrentProjectFind::CurrentProjectFind()
 {
     connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
             this, &CurrentProjectFind::handleProjectChanged);
+    connect(SessionManager::instance(), &SessionManager::projectDisplayNameChanged,
+            this, [this](ProjectExplorer::Project *p) {
+        if (p == ProjectTree::currentProject())
+            emit displayNameChanged();
+    });
 }
 
 QString CurrentProjectFind::id() const
@@ -73,14 +78,15 @@ QVariant CurrentProjectFind::additionalParameters() const
 }
 
 Utils::FileIterator *CurrentProjectFind::files(const QStringList &nameFilters,
-                           const QVariant &additionalParameters) const
+                                               const QStringList &exclusionFilters,
+                                               const QVariant &additionalParameters) const
 {
     QTC_ASSERT(additionalParameters.isValid(),
                return new Utils::FileListIterator(QStringList(), QList<QTextCodec *>()));
     QString projectFile = additionalParameters.toString();
-    foreach (Project *project, SessionManager::projects()) {
+    for (Project *project : SessionManager::projects()) {
         if (project->document() && projectFile == project->projectFilePath().toString())
-            return filesForProjects(nameFilters, QList<Project *>() << project);
+            return filesForProjects(nameFilters, exclusionFilters, QList<Project *>() << project);
     }
     return new Utils::FileListIterator(QStringList(), QList<QTextCodec *>());
 }
@@ -100,11 +106,11 @@ void CurrentProjectFind::handleProjectChanged()
 
 void CurrentProjectFind::recheckEnabled()
 {
-    Core::SearchResult *search = qobject_cast<Core::SearchResult *>(sender());
+    auto search = qobject_cast<Core::SearchResult *>(sender());
     if (!search)
         return;
     QString projectFile = getAdditionalParameters(search).toString();
-    foreach (Project *project, SessionManager::projects()) {
+    for (Project *project : SessionManager::projects()) {
         if (projectFile == project->projectFilePath().toString()) {
             search->setSearchAgainEnabled(true);
             return;
@@ -123,6 +129,6 @@ void CurrentProjectFind::writeSettings(QSettings *settings)
 void CurrentProjectFind::readSettings(QSettings *settings)
 {
     settings->beginGroup(QLatin1String("CurrentProjectFind"));
-    readCommonSettings(settings, QString(QLatin1Char('*')));
+    readCommonSettings(settings, "*", "");
     settings->endGroup();
 }
