@@ -37,10 +37,15 @@
 #include "settingsaccessor.h"
 
 #include <coreplugin/idocument.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/iversioncontrol.h>
+#include <coreplugin/vcsmanager.h>
+
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/kitmanager.h>
+#include <projectexplorer/projecttree.h>
 
 #include <utils/algorithm.h>
 #include <utils/macroexpander.h>
@@ -79,19 +84,55 @@ const char PLUGIN_SETTINGS_KEY[] = "ProjectExplorer.Project.PluginSettings";
 } // namespace
 
 namespace ProjectExplorer {
+
+// --------------------------------------------------------------------
+// ProjectDocument:
+// --------------------------------------------------------------------
+
+ProjectDocument::ProjectDocument(const QString &mimeType, const Utils::FileName &fileName,
+                                 const ProjectDocument::ProjectCallback &callback) :
+    m_callback(callback)
+{
+    setFilePath(fileName);
+    setMimeType(mimeType);
+    if (m_callback)
+        Core::DocumentManager::addDocument(this);
+}
+
+Core::IDocument::ReloadBehavior
+ProjectDocument::reloadBehavior(Core::IDocument::ChangeTrigger state,
+                                Core::IDocument::ChangeType type) const
+{
+    Q_UNUSED(state);
+    Q_UNUSED(type);
+    return BehaviorSilent;
+}
+
+bool ProjectDocument::reload(QString *errorString, Core::IDocument::ReloadFlag flag,
+                             Core::IDocument::ChangeType type)
+{
+    Q_UNUSED(errorString);
+    Q_UNUSED(flag);
+    Q_UNUSED(type);
+
+    if (m_callback)
+        m_callback();
+    return true;
+}
+
 // -------------------------------------------------------------------------
 // Project
 // -------------------------------------------------------------------------
-
 class ProjectPrivate
 {
 public:
+    ProjectPrivate(Core::IDocument *document) : m_document(document) { }
     ~ProjectPrivate();
 
     Core::Id m_id;
     Core::IDocument *m_document = nullptr;
-    IProjectManager *m_manager = nullptr;
     ProjectNode *m_rootProjectNode = nullptr;
+    ContainerNode *m_containerNode = nullptr;
     QList<Target *> m_targets;
     Target *m_activeTarget = nullptr;
     EditorConfiguration m_editorConfiguration;
@@ -100,8 +141,10 @@ public:
     QVariantMap m_pluginSettings;
     Internal::UserFileAccessor *m_accessor = nullptr;
 
-    KitMatcher m_requiredKitMatcher;
-    KitMatcher m_preferredKitMatcher;
+    QString m_displayName;
+
+    Kit::Predicate m_requiredKitPredicate;
+    Kit::Predicate m_preferredKitPredicate;
 
     Utils::MacroExpander m_macroExpander;
 };
@@ -113,15 +156,22 @@ ProjectPrivate::~ProjectPrivate()
     m_rootProjectNode = nullptr;
     delete oldNode;
 
+    delete m_containerNode;
+
     delete m_document;
     delete m_accessor;
 }
 
-Project::Project() : d(new ProjectPrivate)
+Project::Project(const QString &mimeType, const Utils::FileName &fileName,
+                 const ProjectDocument::ProjectCallback &callback) :
+    d(new ProjectPrivate(new ProjectDocument(mimeType, fileName, callback)))
 {
     d->m_macroExpander.setDisplayName(tr("Project"));
     d->m_macroExpander.registerVariable("Project:Name", tr("Project Name"),
             [this] { return displayName(); });
+
+    // Only set up containernode after d is set so that it will find the project directory!
+    d->m_containerNode = new ContainerNode(this);
 }
 
 Project::~Project()
@@ -130,6 +180,11 @@ Project::~Project()
     delete d;
 }
 
+QString Project::displayName() const
+{
+    return d->m_displayName;
+}
+
 Core::Id Project::id() const
 {
     QTC_CHECK(d->m_id.isValid());
@@ -274,10 +329,10 @@ Target *Project::createTarget(Kit *k)
     return t;
 }
 
-Target *Project::cloneTarget(Target *sourceTarget, Kit *k)
+bool Project::copySteps(Target *sourceTarget, Target *newTarget)
 {
-    auto newTarget = new Target(this, k);
-
+    QTC_ASSERT(newTarget, return false);
+    bool fatalError = false;
     QStringList buildconfigurationError;
     QStringList deployconfigurationError;
     QStringList runconfigurationError;
@@ -348,7 +403,6 @@ Target *Project::cloneTarget(Target *sourceTarget, Kit *k)
             newTarget->setActiveRunConfiguration(rcs.first());
     }
 
-    bool fatalError = false;
     if (buildconfigurationError.count() == sourceTarget->buildConfigurations().count())
         fatalError = true;
 
@@ -364,10 +418,7 @@ Target *Project::cloneTarget(Target *sourceTarget, Kit *k)
                               tr("Incompatible Kit"),
                               tr("Kit %1 is incompatible with kit %2.")
                               .arg(sourceTarget->kit()->displayName())
-                              .arg(k->displayName()));
-
-        delete newTarget;
-        newTarget = nullptr;
+                              .arg(newTarget->kit()->displayName()));
     } else if (!buildconfigurationError.isEmpty()
                || !deployconfigurationError.isEmpty()
                || ! runconfigurationError.isEmpty()) {
@@ -397,13 +448,10 @@ Target *Project::cloneTarget(Target *sourceTarget, Kit *k)
         msgBox.setText(tr("Some configurations could not be copied."));
         msgBox.setDetailedText(error);
         msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
-        if (msgBox.exec() != QDialog::Accepted) {
-            delete newTarget;
-            newTarget = nullptr;
-        }
+        fatalError = msgBox.exec() != QDialog::Accepted;
     }
 
-    return newTarget;
+    return !fatalError;
 }
 
 bool Project::setupTarget(Target *t)
@@ -414,29 +462,44 @@ bool Project::setupTarget(Target *t)
     return true;
 }
 
+void Project::setDisplayName(const QString &name)
+{
+    if (name == d->m_displayName)
+        return;
+    d->m_displayName = name;
+    emit displayNameChanged();
+}
+
 void Project::setId(Core::Id id)
 {
     d->m_id = id;
 }
 
-void Project::setDocument(Core::IDocument *doc)
-{
-    QTC_ASSERT(doc, return);
-    QTC_ASSERT(!d->m_document, return);
-    d->m_document = doc;
-}
-
-void Project::setProjectManager(IProjectManager *manager)
-{
-    QTC_ASSERT(manager, return);
-    QTC_ASSERT(!d->m_manager, return);
-    d->m_manager = manager;
-}
-
 void Project::setRootProjectNode(ProjectNode *root)
 {
+    if (d->m_rootProjectNode == root)
+        return;
+
+    if (root && root->nodes().isEmpty()) {
+        // Something went wrong with parsing: At least the project file needs to be
+        // shown so that the user can fix the breakage.
+        // Do not leak root and use default project tree in this case.
+        delete root;
+        root = nullptr;
+    }
+
+    ProjectTree::applyTreeManager(root);
+
     ProjectNode *oldNode = d->m_rootProjectNode;
     d->m_rootProjectNode = root;
+    if (root) {
+        root->setParentFolderNode(d->m_containerNode);
+        // Only announce non-null root, null is only used when project is destroyed.
+        // In that case SessionManager::projectRemoved() triggers the update.
+        ProjectTree::emitSubtreeChanged(root);
+        emit fileListChanged();
+    }
+
     delete oldNode;
 }
 
@@ -449,7 +512,7 @@ Target *Project::restoreTarget(const QVariantMap &data)
         return nullptr;
     }
 
-    Kit *k = KitManager::find(id);
+    Kit *k = KitManager::kit(id);
     if (!k) {
         qWarning("Warning: No kit '%s' found. Continuing.", qPrintable(id.toString()));
         return nullptr;
@@ -484,6 +547,33 @@ Project::RestoreResult Project::restoreSettings(QString *errorMessage)
     return result;
 }
 
+QStringList Project::files(Project::FilesMode fileMode,
+                           const std::function<bool(const Node *)> &filter) const
+{
+    QStringList result;
+
+    if (!rootProjectNode())
+        return result;
+
+    QSet<QString> alreadySeen;
+    rootProjectNode()->forEachGenericNode([&](const Node *n) {
+        const QString path = n->filePath().toString();
+        const int count = alreadySeen.count();
+        alreadySeen.insert(path);
+        if (count == alreadySeen.count())
+            return; // skip duplicates
+        if (!n->listInProject())
+            return;
+        if (filter && !filter(n))
+            return;
+        if ((fileMode == AllFiles)
+                || (fileMode == SourceFiles && !n->isGenerated())
+                || (fileMode == GeneratedFiles && n->isGenerated()))
+            result.append(path);
+    });
+    return result;
+}
+
 
 /*!
     Serializes all data into a QVariantMap.
@@ -536,17 +626,16 @@ Utils::FileName Project::projectDirectory(const Utils::FileName &top)
     return Utils::FileName::fromString(top.toFileInfo().absoluteDir().path());
 }
 
-IProjectManager *Project::projectManager() const
-{
-    QTC_CHECK(d->m_manager);
-    return d->m_manager;
-}
-
 ProjectNode *Project::rootProjectNode() const
 {
     return d->m_rootProjectNode;
 }
 
+ContainerNode *Project::containerNode() const
+{
+    return d->m_containerNode;
+}
+
 Project::RestoreResult Project::fromMap(const QVariantMap &map, QString *errorMessage)
 {
     Q_UNUSED(errorMessage);
@@ -693,7 +782,7 @@ void Project::setup(QList<const BuildInfo *> infoList)
 {
     QList<Target *> toRegister;
     foreach (const BuildInfo *info, infoList) {
-        Kit *k = KitManager::find(info->kitId);
+        Kit *k = KitManager::kit(info->kitId);
         if (!k)
             continue;
         Target *t = target(k);
@@ -722,29 +811,29 @@ Utils::MacroExpander *Project::macroExpander() const
     return &d->m_macroExpander;
 }
 
-ProjectImporter *Project::createProjectImporter() const
+ProjectImporter *Project::projectImporter() const
 {
     return nullptr;
 }
 
-KitMatcher Project::requiredKitMatcher() const
+Kit::Predicate Project::requiredKitPredicate() const
 {
-    return d->m_requiredKitMatcher;
+    return d->m_requiredKitPredicate;
 }
 
-void Project::setRequiredKitMatcher(const KitMatcher &matcher)
+void Project::setRequiredKitPredicate(const Kit::Predicate &predicate)
 {
-    d->m_requiredKitMatcher = matcher;
+    d->m_requiredKitPredicate = predicate;
 }
 
-KitMatcher Project::preferredKitMatcher() const
+Kit::Predicate Project::preferredKitPredicate() const
 {
-    return d->m_preferredKitMatcher;
+    return d->m_preferredKitPredicate;
 }
 
-void Project::setPreferredKitMatcher(const KitMatcher &matcher)
+void Project::setPreferredKitPredicate(const Kit::Predicate &predicate)
 {
-    d->m_preferredKitMatcher = matcher;
+    d->m_preferredKitPredicate = predicate;
 }
 
 void Project::onBuildDirectoryChanged()
