@@ -26,6 +26,7 @@
 #include "qbsinstallstep.h"
 
 #include "qbsbuildconfiguration.h"
+#include "qbsbuildstep.h"
 #include "qbsproject.h"
 #include "qbsprojectmanagerconstants.h"
 
@@ -45,7 +46,6 @@
 // Constants:
 // --------------------------------------------------------------------
 
-static const char QBS_INSTALL_ROOT[] = "Qbs.InstallRoot";
 static const char QBS_REMOVE_FIRST[] = "Qbs.RemoveFirst";
 static const char QBS_DRY_RUN[] = "Qbs.DryRun";
 static const char QBS_KEEP_GOING[] = "Qbs.DryKeepGoing";
@@ -62,13 +62,16 @@ QbsInstallStep::QbsInstallStep(ProjectExplorer::BuildStepList *bsl) :
     m_job(0), m_showCompilerOutput(true), m_parser(0)
 {
     setDisplayName(tr("Qbs Install"));
+    ctor();
 }
 
 QbsInstallStep::QbsInstallStep(ProjectExplorer::BuildStepList *bsl, const QbsInstallStep *other) :
     ProjectExplorer::BuildStep(bsl, Core::Id(Constants::QBS_INSTALLSTEP_ID)),
     m_qbsInstallOptions(other->m_qbsInstallOptions), m_job(0),
     m_showCompilerOutput(other->m_showCompilerOutput), m_parser(0)
-{ }
+{
+    ctor();
+}
 
 QbsInstallStep::~QbsInstallStep()
 {
@@ -93,18 +96,17 @@ void QbsInstallStep::run(QFutureInterface<bool> &fi)
     m_job = pro->install(m_qbsInstallOptions);
 
     if (!m_job) {
-        m_fi->reportResult(false);
-        emit finished();
+        reportRunResult(*m_fi, false);
         return;
     }
 
     m_progressBase = 0;
 
-    connect(m_job, SIGNAL(finished(bool,qbs::AbstractJob*)), this, SLOT(installDone(bool)));
-    connect(m_job, SIGNAL(taskStarted(QString,int,qbs::AbstractJob*)),
-            this, SLOT(handleTaskStarted(QString,int)));
-    connect(m_job, SIGNAL(taskProgress(int,qbs::AbstractJob*)),
-            this, SLOT(handleProgress(int)));
+    connect(m_job, &qbs::AbstractJob::finished, this, &QbsInstallStep::installDone);
+    connect(m_job, &qbs::AbstractJob::taskStarted,
+            this, &QbsInstallStep::handleTaskStarted);
+    connect(m_job, &qbs::AbstractJob::taskProgress,
+            this, &QbsInstallStep::handleProgress);
 }
 
 ProjectExplorer::BuildStepConfigWidget *QbsInstallStep::createConfigWidget()
@@ -125,19 +127,8 @@ void QbsInstallStep::cancel()
 
 QString QbsInstallStep::installRoot() const
 {
-    if (!m_qbsInstallOptions.installRoot().isEmpty())
-        return m_qbsInstallOptions.installRoot();
-
-    return qbs::InstallOptions::defaultInstallRoot();
-}
-
-QString QbsInstallStep::absoluteInstallRoot() const
-{
-    const qbs::ProjectData data = static_cast<QbsProject *>(project())->qbsProjectData();
-    QString path = installRoot();
-    if (data.isValid() && !data.buildDirectory().isEmpty() && !path.isEmpty())
-        path = QDir(data.buildDirectory()).absoluteFilePath(path);
-    return path;
+    const QbsBuildStep * const bs = buildConfig()->qbsStep();
+    return bs ? bs->installRoot().toString() : QString();
 }
 
 bool QbsInstallStep::removeFirst() const
@@ -155,13 +146,31 @@ bool QbsInstallStep::keepGoing() const
     return m_qbsInstallOptions.keepGoing();
 }
 
+void QbsInstallStep::ctor()
+{
+    const QbsBuildConfiguration * const bc = buildConfig();
+    connect(bc, &QbsBuildConfiguration::qbsConfigurationChanged,
+            this, &QbsInstallStep::handleBuildConfigChanged);
+    if (bc->qbsStep()) {
+        connect(bc->qbsStep(), &QbsBuildStep::qbsBuildOptionsChanged,
+                this, &QbsInstallStep::handleBuildConfigChanged);
+    }
+}
+
+const QbsBuildConfiguration *QbsInstallStep::buildConfig() const
+{
+    return static_cast<QbsBuildConfiguration *>(
+                deployConfiguration()->target()->activeBuildConfiguration());
+}
+
 bool QbsInstallStep::fromMap(const QVariantMap &map)
 {
     if (!ProjectExplorer::BuildStep::fromMap(map))
         return false;
 
-    setInstallRoot(map.value(QLatin1String(QBS_INSTALL_ROOT)).toString());
-    m_qbsInstallOptions.setRemoveExistingInstallation(map.value(QLatin1String(QBS_REMOVE_FIRST), false).toBool());
+    m_qbsInstallOptions.setInstallRoot(installRoot());
+    m_qbsInstallOptions.setRemoveExistingInstallation(
+                map.value(QLatin1String(QBS_REMOVE_FIRST), false).toBool());
     m_qbsInstallOptions.setDryRun(map.value(QLatin1String(QBS_DRY_RUN), false).toBool());
     m_qbsInstallOptions.setKeepGoing(map.value(QLatin1String(QBS_KEEP_GOING), false).toBool());
 
@@ -171,11 +180,9 @@ bool QbsInstallStep::fromMap(const QVariantMap &map)
 QVariantMap QbsInstallStep::toMap() const
 {
     QVariantMap map = ProjectExplorer::BuildStep::toMap();
-    map.insert(QLatin1String(QBS_INSTALL_ROOT), m_qbsInstallOptions.installRoot());
     map.insert(QLatin1String(QBS_REMOVE_FIRST), m_qbsInstallOptions.removeExistingInstallation());
     map.insert(QLatin1String(QBS_DRY_RUN), m_qbsInstallOptions.dryRun());
     map.insert(QLatin1String(QBS_KEEP_GOING), m_qbsInstallOptions.keepGoing());
-
     return map;
 }
 
@@ -193,12 +200,10 @@ void QbsInstallStep::installDone(bool success)
     }
 
     QTC_ASSERT(m_fi, return);
-    m_fi->reportResult(success);
+    reportRunResult(*m_fi, success);
     m_fi = 0; // do not delete, it is not ours
     m_job->deleteLater();
     m_job = 0;
-
-    emit finished();
 }
 
 void QbsInstallStep::handleTaskStarted(const QString &desciption, int max)
@@ -222,15 +227,7 @@ void QbsInstallStep::createTaskAndOutput(ProjectExplorer::Task::TaskType type,
                                                        Utils::FileName::fromString(file), line,
                                                        ProjectExplorer::Constants::TASK_CATEGORY_COMPILE);
     emit addTask(task, 1);
-    emit addOutput(message, NormalOutput);
-}
-
-void QbsInstallStep::setInstallRoot(const QString &ir)
-{
-    if (m_qbsInstallOptions.installRoot() == ir)
-        return;
-    m_qbsInstallOptions.setInstallRoot(ir);
-    emit changed();
+    emit addOutput(message, OutputFormat::Stdout);
 }
 
 void QbsInstallStep::setRemoveFirst(bool rf)
@@ -257,6 +254,12 @@ void QbsInstallStep::setKeepGoing(bool kg)
     emit changed();
 }
 
+void QbsInstallStep::handleBuildConfigChanged()
+{
+    m_qbsInstallOptions.setInstallRoot(installRoot());
+    emit changed();
+}
+
 // --------------------------------------------------------------------
 // QbsInstallStepConfigWidget:
 // --------------------------------------------------------------------
@@ -264,8 +267,10 @@ void QbsInstallStep::setKeepGoing(bool kg)
 QbsInstallStepConfigWidget::QbsInstallStepConfigWidget(QbsInstallStep *step) :
     m_step(step), m_ignoreChange(false)
 {
-    connect(m_step, SIGNAL(displayNameChanged()), this, SLOT(updateState()));
-    connect(m_step, SIGNAL(changed()), this, SLOT(updateState()));
+    connect(m_step, &ProjectExplorer::ProjectConfiguration::displayNameChanged,
+            this, &QbsInstallStepConfigWidget::updateState);
+    connect(m_step, &QbsInstallStep::changed,
+            this, &QbsInstallStepConfigWidget::updateState);
 
     setContentsMargins(0, 0, 0, 0);
 
@@ -274,17 +279,15 @@ QbsInstallStepConfigWidget::QbsInstallStepConfigWidget(QbsInstallStep *step) :
     m_ui = new Ui::QbsInstallStepConfigWidget;
     m_ui->setupUi(this);
 
-    m_ui->installRootChooser->setPromptDialogTitle(tr("Qbs Install Prefix"));
-    m_ui->installRootChooser->setExpectedKind(Utils::PathChooser::Directory);
-    m_ui->installRootChooser->setHistoryCompleter(QLatin1String("Qbs.InstallRoot.History"));
+    connect(m_ui->removeFirstCheckBox, &QAbstractButton::toggled,
+            this, &QbsInstallStepConfigWidget::changeRemoveFirst);
+    connect(m_ui->dryRunCheckBox, &QAbstractButton::toggled,
+            this, &QbsInstallStepConfigWidget::changeDryRun);
+    connect(m_ui->keepGoingCheckBox, &QAbstractButton::toggled,
+            this, &QbsInstallStepConfigWidget::changeKeepGoing);
 
-    connect(m_ui->installRootChooser, SIGNAL(rawPathChanged(QString)), this,
-            SLOT(changeInstallRoot()));
-    connect(m_ui->removeFirstCheckBox, SIGNAL(toggled(bool)), this, SLOT(changeRemoveFirst(bool)));
-    connect(m_ui->dryRunCheckBox, SIGNAL(toggled(bool)), this, SLOT(changeDryRun(bool)));
-    connect(m_ui->keepGoingCheckBox, SIGNAL(toggled(bool)), this, SLOT(changeKeepGoing(bool)));
-
-    connect(project, SIGNAL(projectParsingDone(bool)), this, SLOT(updateState()));
+    connect(project, &QbsProject::projectParsingDone,
+            this, &QbsInstallStepConfigWidget::updateState);
 
     updateState();
 }
@@ -307,15 +310,14 @@ QString QbsInstallStepConfigWidget::displayName() const
 void QbsInstallStepConfigWidget::updateState()
 {
     if (!m_ignoreChange) {
-        m_ui->installRootChooser->setPath(m_step->installRoot());
+        m_ui->installRootValueLabel->setText(m_step->installRoot());
         m_ui->removeFirstCheckBox->setChecked(m_step->removeFirst());
         m_ui->dryRunCheckBox->setChecked(m_step->dryRun());
         m_ui->keepGoingCheckBox->setChecked(m_step->keepGoing());
     }
 
-    m_ui->installRootChooser->setBaseFileName(m_step->target()->activeBuildConfiguration()->buildDirectory());
+    QString command = m_step->buildConfig()->equivalentCommandLine(m_step);
 
-    QString command = QbsBuildConfiguration::equivalentCommandLine(m_step);
     m_ui->commandLineTextEdit->setPlainText(command);
 
     QString summary = tr("<b>Qbs:</b> %1").arg(command);
@@ -325,17 +327,6 @@ void QbsInstallStepConfigWidget::updateState()
     }
 }
 
-void QbsInstallStepConfigWidget::changeInstallRoot()
-{
-    const QString path = m_ui->installRootChooser->path();
-    if (m_step->installRoot() == path)
-        return;
-
-    m_ignoreChange = true;
-    m_step->setInstallRoot(path);
-    m_ignoreChange = false;
-}
-
 void QbsInstallStepConfigWidget::changeRemoveFirst(bool rf)
 {
     m_step->setRemoveFirst(rf);
@@ -359,68 +350,25 @@ QbsInstallStepFactory::QbsInstallStepFactory(QObject *parent) :
     ProjectExplorer::IBuildStepFactory(parent)
 { }
 
-QList<Core::Id> QbsInstallStepFactory::availableCreationIds(ProjectExplorer::BuildStepList *parent) const
+QList<ProjectExplorer::BuildStepInfo> QbsInstallStepFactory::availableSteps(ProjectExplorer::BuildStepList *parent) const
 {
     if (parent->id() == ProjectExplorer::Constants::BUILDSTEPS_DEPLOY
             && qobject_cast<ProjectExplorer::DeployConfiguration *>(parent->parent())
             && qobject_cast<QbsProject *>(parent->target()->project()))
-        return QList<Core::Id>() << Core::Id(Constants::QBS_INSTALLSTEP_ID);
-    return QList<Core::Id>();
-}
-
-QString QbsInstallStepFactory::displayNameForId(Core::Id id) const
-{
-    if (id == Core::Id(Constants::QBS_INSTALLSTEP_ID))
-        return tr("Qbs Install");
-    return QString();
-}
-
-bool QbsInstallStepFactory::canCreate(ProjectExplorer::BuildStepList *parent, Core::Id id) const
-{
-    if (parent->id() != Core::Id(ProjectExplorer::Constants::BUILDSTEPS_DEPLOY)
-            || !qobject_cast<ProjectExplorer::DeployConfiguration *>(parent->parent())
-            || !qobject_cast<QbsProject *>(parent->target()->project()))
-        return false;
-    return id == Core::Id(Constants::QBS_INSTALLSTEP_ID);
+        return {{ Constants::QBS_INSTALLSTEP_ID, tr("Qbs Install") }};
+    return {};
 }
 
 ProjectExplorer::BuildStep *QbsInstallStepFactory::create(ProjectExplorer::BuildStepList *parent,
                                                           const Core::Id id)
 {
-    if (!canCreate(parent, id))
-        return 0;
+    Q_UNUSED(id);
     return new QbsInstallStep(parent);
 }
 
-bool QbsInstallStepFactory::canRestore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map) const
-{
-    return canCreate(parent, ProjectExplorer::idFromMap(map));
-}
-
-ProjectExplorer::BuildStep *QbsInstallStepFactory::restore(ProjectExplorer::BuildStepList *parent,
-                                                           const QVariantMap &map)
-{
-    if (!canRestore(parent, map))
-        return 0;
-    QbsInstallStep *bs = new QbsInstallStep(parent);
-    if (!bs->fromMap(map)) {
-        delete bs;
-        return 0;
-    }
-    return bs;
-}
-
-bool QbsInstallStepFactory::canClone(ProjectExplorer::BuildStepList *parent,
-                                     ProjectExplorer::BuildStep *product) const
-{
-    return canCreate(parent, product->id());
-}
-
 ProjectExplorer::BuildStep *QbsInstallStepFactory::clone(ProjectExplorer::BuildStepList *parent,
                                                          ProjectExplorer::BuildStep *product)
 {
-    if (!canClone(parent, product))
-        return 0;
     return new QbsInstallStep(parent, static_cast<QbsInstallStep *>(product));
 }
 
