@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef DesignDocument_h
-#define DesignDocument_h
+#pragma once
 
 #include <model.h>
 #include <rewriterview.h>
@@ -43,12 +37,8 @@
 #include <QStackedWidget>
 
 QT_BEGIN_NAMESPACE
-class QUndoStack;
 class QWidget;
-class QIODevice;
-class QProcess;
 class QPlainTextEdit;
-class QDeclarativeError;
 QT_END_NAMESPACE
 
 namespace ProjectExplorer {
@@ -86,8 +76,10 @@ public:
     Model *documentModel() const;
 
     QString contextHelpId() const;
-    QList<RewriterView::Error> qmlSyntaxErrors() const;
-    bool hasQmlSyntaxErrors() const;
+    QList<DocumentMessage> qmlParseWarnings() const;
+    bool hasQmlParseWarnings() const;
+    QList<DocumentMessage> qmlParseErrors() const;
+    bool hasQmlParseErrors() const;
 
     RewriterView *rewriterView() const;
 
@@ -96,7 +88,7 @@ public:
 
     TextEditor::BaseTextEditor *textEditor() const;
     QPlainTextEdit *plainTextEdit() const;
-    QString fileName() const;
+    Utils::FileName fileName() const;
     ProjectExplorer::Kit *currentKit() const;
     bool isDocumentLoaded() const;
 
@@ -111,9 +103,9 @@ signals:
     void undoAvailable(bool isAvailable);
     void redoAvailable(bool isAvailable);
     void designDocumentClosed();
-    void qmlErrorsChanged(const QList<RewriterView::Error> &errors);
+    void qmlErrorsChanged(const QList<DocumentMessage> &errors);
 
-public slots:
+public:
     void deleteSelected();
     void copySelected();
     void cutSelected();
@@ -122,15 +114,17 @@ public slots:
     void undo();
     void redo();
     void updateActiveQtVersion();
+    void updateCurrentProject();
     void changeToSubComponent(const ModelNode &componentNode);
     void changeToMaster();
 
-private slots:
-    void updateFileName(const QString &oldFileName, const QString &newFileName);
-
 private: // functions
+    void updateFileName(const Utils::FileName &oldFileName, const Utils::FileName &newFileName);
+
     void changeToInFileComponentModel(ComponentTextModifier *textModifer);
 
+    void updateQrcFiles();
+
     QWidget *centralWidget() const;
     QString pathToQt() const;
 
@@ -160,6 +154,3 @@ private: // variables
 };
 
 } // namespace QmlDesigner
-
-
-#endif // DesignDocument_h
