@@ -38,6 +38,7 @@
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
@@ -59,6 +60,8 @@
 #include <QTextStream>
 #include <QTimer>
 #include <QRegExp>
+#include <QSet>
+#include <QString>
 #include <QLibraryInfo>
 #include <qglobal.h>
 
@@ -74,21 +77,22 @@ ModelManagerInterface::ProjectInfo ModelManager::defaultProjectInfoForProject(
         Project *project) const
 {
     ModelManagerInterface::ProjectInfo projectInfo(project);
-    Target *activeTarget = 0;
+    projectInfo.qmlDumpEnvironment = Utils::Environment::systemEnvironment();
+    Target *activeTarget = nullptr;
     if (project) {
-        MimeDatabase mdb;
-        QSet<QString> qmlTypeNames;
-        qmlTypeNames << QLatin1String(Constants::QML_MIMETYPE)
-                     << QLatin1String(Constants::QBS_MIMETYPE)
-                     << QLatin1String(Constants::QMLPROJECT_MIMETYPE)
-                     << QLatin1String(Constants::QMLTYPES_MIMETYPE)
-                     << QLatin1String(Constants::QMLUI_MIMETYPE);
-        foreach (const QString &filePath, project->files(Project::SourceFiles)) {
-            if (qmlTypeNames.contains(mdb.mimeTypeForFile(
-                                          filePath, MimeDatabase::MatchExtension).name())) {
-                projectInfo.sourceFiles << filePath;
+        const QSet<QString> qmlTypeNames = { Constants::QML_MIMETYPE ,Constants::QBS_MIMETYPE,
+                                             Constants::QMLPROJECT_MIMETYPE,
+                                             Constants::QMLTYPES_MIMETYPE,
+                                             Constants::QMLUI_MIMETYPE };
+        projectInfo.sourceFiles = project->files(Project::SourceFiles,
+                                                 [&qmlTypeNames](const Node *n) {
+            if (const FileNode *fn = n->asFileNode()) {
+                return fn->fileType() == FileType::QML
+                        && qmlTypeNames.contains(Utils::mimeTypeForFile(fn->filePath().toString(),
+                                                                        MimeMatchMode::MatchExtension).name());
             }
-        }
+            return false;
+        });
         activeTarget = project->activeTarget();
     }
     Kit *activeKit = activeTarget ? activeTarget->kit() : KitManager::defaultKit();
@@ -102,6 +106,10 @@ ModelManagerInterface::ProjectInfo ModelManager::defaultProjectInfoForProject(
         if (BuildConfiguration *bc = activeTarget->activeBuildConfiguration()) {
             preferDebugDump = bc->buildType() == BuildConfiguration::Debug;
             setPreferDump = true;
+            // Append QML2_IMPORT_PATH if it is defined in build configuration.
+            // It enables qmlplugindump to correctly dump custom plugins or other dependent
+            // plugins that are not installed in default Qt qml installation directory.
+            projectInfo.qmlDumpEnvironment.appendOrSet("QML2_IMPORT_PATH", bc->environment().value("QML2_IMPORT_PATH"), ":");
         }
     }
     if (!setPreferDump && qtVersion)
@@ -118,7 +126,7 @@ ModelManagerInterface::ProjectInfo ModelManager::defaultProjectInfoForProject(
     }
 
     if (projectInfo.tryQmlDump) {
-        QtSupport::QmlDumpTool::pathAndEnvironment(qtVersion,
+        QtSupport::QmlDumpTool::pathAndEnvironment(activeKit,
                                                    preferDebugDump, &projectInfo.qmlDumpPath,
                                                    &projectInfo.qmlDumpEnvironment);
         projectInfo.qmlDumpHasRelocatableFlag = qtVersion->hasQmlDumpWithRelocatableFlag();
@@ -168,34 +176,40 @@ void setupProjectInfoQmlBundles(ModelManagerInterface::ProjectInfo &projectInfo)
 
 namespace Internal {
 
-QHash<QString,Dialect> ModelManager::languageForSuffix() const
+
+QHash<QString,Dialect> ModelManager::initLanguageForSuffix() const
 {
     QHash<QString,Dialect> res = ModelManagerInterface::languageForSuffix();
 
     if (ICore::instance()) {
-        MimeDatabase mdb;
-        MimeType jsSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::JS_MIMETYPE));
+        MimeType jsSourceTy = Utils::mimeTypeForName(Constants::JS_MIMETYPE);
         foreach (const QString &suffix, jsSourceTy.suffixes())
             res[suffix] = Dialect::JavaScript;
-        MimeType qmlSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::QML_MIMETYPE));
+        MimeType qmlSourceTy = Utils::mimeTypeForName(Constants::QML_MIMETYPE);
         foreach (const QString &suffix, qmlSourceTy.suffixes())
             res[suffix] = Dialect::Qml;
-        MimeType qbsSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::QBS_MIMETYPE));
+        MimeType qbsSourceTy = Utils::mimeTypeForName(Constants::QBS_MIMETYPE);
         foreach (const QString &suffix, qbsSourceTy.suffixes())
             res[suffix] = Dialect::QmlQbs;
-        MimeType qmlProjectSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::QMLPROJECT_MIMETYPE));
+        MimeType qmlProjectSourceTy = Utils::mimeTypeForName(Constants::QMLPROJECT_MIMETYPE);
         foreach (const QString &suffix, qmlProjectSourceTy.suffixes())
             res[suffix] = Dialect::QmlProject;
-        MimeType qmlUiSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::QMLUI_MIMETYPE));
+        MimeType qmlUiSourceTy = Utils::mimeTypeForName(Constants::QMLUI_MIMETYPE);
         foreach (const QString &suffix, qmlUiSourceTy.suffixes())
             res[suffix] = Dialect::QmlQtQuick2Ui;
-        MimeType jsonSourceTy = mdb.mimeTypeForName(QLatin1String(Constants::JSON_MIMETYPE));
+        MimeType jsonSourceTy = Utils::mimeTypeForName(Constants::JSON_MIMETYPE);
         foreach (const QString &suffix, jsonSourceTy.suffixes())
             res[suffix] = Dialect::Json;
     }
     return res;
 }
 
+QHash<QString,Dialect> ModelManager::languageForSuffix() const
+{
+    static QHash<QString,Dialect> res = initLanguageForSuffix();
+    return res;
+}
+
 ModelManager::ModelManager(QObject *parent):
         ModelManagerInterface(parent)
 {
@@ -212,8 +226,8 @@ void ModelManager::delayedInitialization()
     CppTools::CppModelManager *cppModelManager = CppTools::CppModelManager::instance();
     // It's important to have a direct connection here so we can prevent
     // the source and AST of the cpp document being cleaned away.
-    connect(cppModelManager, SIGNAL(documentUpdated(CPlusPlus::Document::Ptr)),
-            this, SLOT(maybeQueueCppQmlTypeUpdate(CPlusPlus::Document::Ptr)), Qt::DirectConnection);
+    connect(cppModelManager, &CppTools::CppModelManager::documentUpdated,
+            this, &ModelManagerInterface::maybeQueueCppQmlTypeUpdate, Qt::DirectConnection);
 
     connect(SessionManager::instance(), &SessionManager::projectRemoved,
             this, &ModelManager::removeProjectInfo);
@@ -242,12 +256,19 @@ void ModelManager::writeMessageInternal(const QString &msg) const
 ModelManagerInterface::WorkingCopy ModelManager::workingCopyInternal() const
 {
     WorkingCopy workingCopy;
+
+    if (!Core::ICore::instance())
+        return workingCopy;
+
     foreach (IDocument *document, DocumentModel::openedDocuments()) {
         const QString key = document->filePath().toString();
         if (TextEditor::TextDocument *textDocument = qobject_cast<TextEditor::TextDocument *>(document)) {
             // TODO the language should be a property on the document, not the editor
-            if (DocumentModel::editorsForDocument(document).first()->context().contains(ProjectExplorer::Constants::LANG_QMLJS))
-                workingCopy.insert(key, textDocument->plainText(), textDocument->document()->revision());
+            if (DocumentModel::editorsForDocument(document).first()
+                    ->context().contains(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID)) {
+                workingCopy.insert(key, textDocument->plainText(),
+                                   textDocument->document()->revision());
+            }
         }
     }
 
