@@ -27,7 +27,6 @@
 
 #include "qmlprofilermodelmanager.h"
 #include "qmlprofilerstatisticsmodel.h"
-#include "qmlprofilerviewmanager.h"
 #include "qmlprofilereventsview.h"
 #include "qmlprofilereventtypes.h"
 
@@ -80,6 +79,9 @@ public:
     ~QmlProfilerStatisticsView();
     void clear() override;
 
+    QString summary(const QVector<int> &typeIds) const;
+    QStringList details(int typeId) const;
+
 public slots:
     void selectByTypeId(int typeIndex) override;
     void onVisibleFeaturesChanged(quint64 features) override;
