@@ -47,7 +47,6 @@
 #include <utils/runextensions.h>
 #include <utils/synchronousprocess.h>
 #include <utils/winutils.h>
-#include <utils/algorithm.h>
 
 #include <QDir>
 #include <QUrl>
@@ -55,7 +54,9 @@
 #include <QFuture>
 #include <QCoreApplication>
 #include <QProcess>
-#include <QRegExp>
+#include <QRegularExpression>
+
+#include <algorithm>
 
 using namespace Core;
 using namespace QtSupport;
@@ -344,7 +345,7 @@ void BaseQtVersion::setupExpander()
         QtKitInformation::tr("The current Qt version's default mkspecs (Qt 4)."),
         [this] { return qmakeProperty(m_versionInfo, "QMAKE_MKSPECS"); });
     m_expander.registerVariable("Qt:QMAKE_SPEC",
-        QtKitInformation::tr("The current Qt version's default mkspec (Qt 5; host system)"),
+        QtKitInformation::tr("The current Qt version's default mkspec (Qt 5; host system)."),
         [this] { return qmakeProperty(m_versionInfo, "QMAKE_SPEC"); });
     m_expander.registerVariable("Qt:QMAKE_XSPEC",
         QtKitInformation::tr("The current Qt version's default mkspec (Qt 5; target system)."),
@@ -470,6 +471,17 @@ QSet<Id> BaseQtVersion::availableFeatures() const
     if (qtVersion().matches(5, 7))
         return features;
 
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 8));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_2_PREFIX, 2, 1));
+
+    if (qtVersion().matches(5, 8))
+        return features;
+
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 9));
+
+    if (qtVersion().matches(5, 9))
+        return features;
+
     return features;
 }
 
@@ -493,7 +505,7 @@ QList<Task> BaseQtVersion::validateKit(const Kit *k)
                        FileName(), -1, ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM);
     }
 
-    ToolChain *tc = ToolChainKitInformation::toolChain(k);
+    ToolChain *tc = ToolChainKitInformation::toolChain(k, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
     if (tc) {
         Abi targetAbi = tc->targetAbi();
         bool fuzzyMatch = false;
@@ -563,6 +575,22 @@ FileName BaseQtVersion::mkspecsPath() const
     return result;
 }
 
+FileNameList BaseQtVersion::directoriesToIgnoreInProjectTree() const
+{
+    FileNameList result;
+    const FileName mkspecPathGet = mkspecsPath();
+    result.append(mkspecPathGet);
+
+    FileName mkspecPathSrc = FileName::fromUserInput(qmakeProperty("QT_HOST_DATA", PropertyVariantSrc));
+    if (!mkspecPathSrc.isEmpty()) {
+        mkspecPathSrc.appendPath("mkspecs");
+        if (mkspecPathSrc != mkspecPathGet)
+            result.append(mkspecPathSrc);
+    }
+
+    return result;
+}
+
 QString BaseQtVersion::qtNamespace() const
 {
     ensureMkSpecParsed();
@@ -646,6 +674,13 @@ bool BaseQtVersion::isValid() const
             && m_qmakeIsExecutable;
 }
 
+BaseQtVersion::Predicate BaseQtVersion::isValidPredicate(const BaseQtVersion::Predicate &predicate)
+{
+    if (predicate)
+        return [predicate](const BaseQtVersion *v) { return v->isValid() && predicate(v); };
+    return [](const BaseQtVersion *v) { return v->isValid(); };
+}
+
 QString BaseQtVersion::invalidReason() const
 {
     if (displayName().isEmpty())
@@ -669,8 +704,8 @@ QStringList BaseQtVersion::warningReason() const
     QStringList ret;
     if (qtAbis().isEmpty())
         ret << QCoreApplication::translate("QtVersion", "ABI detection failed: Make sure to use a matching compiler when building.");
-    if (m_versionInfo.value(QLatin1String("QT_INSTALL_PREFIX/get"))
-        != m_versionInfo.value(QLatin1String("QT_INSTALL_PREFIX"))) {
+    if (m_versionInfo.value(ProKey("QT_INSTALL_PREFIX/get"))
+        != m_versionInfo.value(ProKey("QT_INSTALL_PREFIX"))) {
         ret << QCoreApplication::translate("QtVersion", "Non-installed -prefix build - for internal development only.");
     }
     return ret;
@@ -784,12 +819,13 @@ QString BaseQtVersion::toHtml(bool verbose) const
         str << "<tr><td><b>" << QCoreApplication::translate("BaseQtVersion", "Version:")
             << "</b></td><td>" << qtVersionString() << "</td></tr>";
         if (verbose) {
-            const QHash<QString,QString> vInfo = versionInfo();
+            const QHash<ProKey, ProString> vInfo = versionInfo();
             if (!vInfo.isEmpty()) {
-                QStringList keys = vInfo.keys();
-                keys.sort();
-                foreach (QString variableName, keys) {
-                    const QString &value = vInfo.value(variableName);
+                QList<ProKey> keys = vInfo.keys();
+                Utils::sort(keys);
+                foreach (const ProKey &key, keys) {
+                    const QString &value = vInfo.value(key).toQString();
+                    QString variableName = key.toQString();
                     if (variableName != QLatin1String("QMAKE_MKSPECS")
                         && !variableName.endsWith(QLatin1String("/raw"))) {
                         bool isPath = false;
@@ -1009,8 +1045,8 @@ void BaseQtVersion::ensureMkSpecParsed() const
         return;
 
     QMakeVfs vfs;
-    ProFileGlobals option;
-    option.setProperties(versionInfo());
+    QMakeGlobals option;
+    applyProperties(&option);
     option.environment = qmakeRunEnvironment().toProcessEnvironment();
     ProMessageHandler msgHandler(true);
     ProFileCacheManager::instance()->incRefCount();
@@ -1082,13 +1118,19 @@ FileName BaseQtVersion::mkspecPath() const
 
 bool BaseQtVersion::hasMkspec(const FileName &spec) const
 {
-    QFileInfo fi;
-    fi.setFile(QDir::fromNativeSeparators(qmakeProperty("QT_HOST_DATA"))
-               + QLatin1String("/mkspecs/") + spec.toString());
-    if (fi.isDir())
+    if (spec.isEmpty())
+        return true; // default spec of a Qt version
+
+    QDir mkspecDir = QDir(QDir::fromNativeSeparators(qmakeProperty("QT_HOST_DATA"))
+                          + QLatin1String("/mkspecs/"));
+    const QString absSpec = mkspecDir.absoluteFilePath(spec.toString());
+    if (QFileInfo(absSpec).isDir() && QFileInfo(absSpec + "/qmake.conf").isFile())
         return true;
-    fi.setFile(sourcePath().toString() + QLatin1String("/mkspecs/") + spec.toString());
-    return fi.isDir();
+    mkspecDir.setPath(sourcePath().toString() + QLatin1String("/mkspecs/"));
+    const QString absSrcSpec = mkspecDir.absoluteFilePath(spec.toString());
+    return absSrcSpec != absSpec
+            && QFileInfo(absSrcSpec).isDir()
+            && QFileInfo(absSrcSpec + "/qmake.conf").isFile();
 }
 
 BaseQtVersion::QmakeBuildConfigs BaseQtVersion::defaultBuildConfig() const
@@ -1182,26 +1224,32 @@ void BaseQtVersion::updateVersionInfo() const
     m_versionInfoUpToDate = true;
 }
 
-QHash<QString,QString> BaseQtVersion::versionInfo() const
+QHash<ProKey,ProString> BaseQtVersion::versionInfo() const
 {
     updateVersionInfo();
     return m_versionInfo;
 }
 
-QString BaseQtVersion::qmakeProperty(const QHash<QString,QString> &versionInfo, const QByteArray &name,
+QString BaseQtVersion::qmakeProperty(const QHash<ProKey,ProString> &versionInfo, const QByteArray &name,
                                      PropertyVariant variant)
 {
-    QString val = versionInfo.value(QString::fromLatin1(
-            name + (variant == PropertyVariantGet ? "/get" : "/src")));
+    QString val = versionInfo.value(ProKey(QString::fromLatin1(
+            name + (variant == PropertyVariantDev ? "/dev" :
+                    variant == PropertyVariantGet ? "/get" : "/src")))).toQString();
     if (!val.isNull())
         return val;
-    return versionInfo.value(QString::fromLatin1(name));
+    return versionInfo.value(ProKey(QString::fromLatin1(name))).toQString();
 }
 
-QString BaseQtVersion::qmakeProperty(const QByteArray &name) const
+QString BaseQtVersion::qmakeProperty(const QByteArray &name, PropertyVariant variant) const
 {
     updateVersionInfo();
-    return qmakeProperty(m_versionInfo, name);
+    return qmakeProperty(m_versionInfo, name, variant);
+}
+
+void BaseQtVersion::applyProperties(QMakeGlobals *qmakeGlobals) const
+{
+    qmakeGlobals->setProperties(versionInfo());
 }
 
 bool BaseQtVersion::hasDocumentation() const
@@ -1295,24 +1343,6 @@ bool BaseQtVersion::needsQmlDump() const
     return qtVersion() < QtVersionNumber(4, 8, 0);
 }
 
-Environment BaseQtVersion::qmlToolsEnvironment() const
-{
-    // FIXME: This seems broken!
-    Environment environment = Environment::systemEnvironment();
-#if 0 // FIXME: Fix this!
-    addToEnvironment(environment);
-#endif
-
-    // add preferred tool chain, as that is how the tools are built, compare QtVersion::buildDebuggingHelperLibrary
-    if (!qtAbis().isEmpty()) {
-        QList<ToolChain *> alltc = ToolChainManager::findToolChains(qtAbis().at(0));
-        if (!alltc.isEmpty())
-            alltc.first()->addToEnvironment(environment);
-    }
-
-    return environment;
-}
-
 QString BaseQtVersion::qmlDumpTool(bool debugVersion) const
 {
     const QString qtInstallBins = qmakeProperty("QT_INSTALL_BINS");
@@ -1409,7 +1439,7 @@ static QByteArray runQmakeQuery(const FileName &binary, const Environment &env,
 }
 
 bool BaseQtVersion::queryQMakeVariables(const FileName &binary, const Environment &env,
-                                        QHash<QString, QString> *versionInfo, QString *error)
+                                        QHash<ProKey, ProString> *versionInfo, QString *error)
 {
     QString tmp;
     if (!error)
@@ -1431,10 +1461,9 @@ bool BaseQtVersion::queryQMakeVariables(const FileName &binary, const Environmen
         // This is required to make non-static qmakes work on windows where every tool chain
         // tries to be incompatible with any other.
         QList<Abi> abiList = Abi::abisOfBinary(binary);
-        QList<ToolChain *> tcList = ToolChainManager::toolChains();
-        foreach (ToolChain *tc, tcList) {
-            if (!abiList.contains(tc->targetAbi()))
-                continue;
+        const QList<ToolChain *> tcList
+                = ToolChainManager::toolChains([&abiList](const ToolChain *t) { return abiList.contains(t->targetAbi()); });
+        for (ToolChain *tc : tcList) {
             Environment realEnv = env;
             tc->addToEnvironment(realEnv);
             output = runQmakeQuery(binary, realEnv, error);
@@ -1446,37 +1475,12 @@ bool BaseQtVersion::queryQMakeVariables(const FileName &binary, const Environmen
     if (output.isNull())
         return false;
 
-    QTextStream stream(&output);
-    while (!stream.atEnd()) {
-        const QString line = stream.readLine();
-        const int index = line.indexOf(QLatin1Char(':'));
-        if (index != -1) {
-            QString name = line.left(index);
-            QString value = QDir::fromNativeSeparators(line.mid(index+1));
-            if (value.isNull())
-                value = QLatin1String(""); // Make sure it is not null, to discern from missing keys
-            versionInfo->insert(name, value);
-            if (name.startsWith(QLatin1String("QT_")) && !name.contains(QLatin1Char('/'))) {
-                if (name.startsWith(QLatin1String("QT_INSTALL_"))) {
-                    versionInfo->insert(name + QLatin1String("/raw"), value);
-                    versionInfo->insert(name + QLatin1String("/get"), value);
-                    if (name == QLatin1String("QT_INSTALL_PREFIX")
-                        || name == QLatin1String("QT_INSTALL_DATA")
-                        || name == QLatin1String("QT_INSTALL_BINS")) {
-                        name.replace(3, 7, QLatin1String("HOST"));
-                        versionInfo->insert(name, value);
-                        versionInfo->insert(name + QLatin1String("/get"), value);
-                    }
-                } else if (name.startsWith(QLatin1String("QT_HOST_"))) {
-                    versionInfo->insert(name + QLatin1String("/get"), value);
-                }
-            }
-        }
-    }
+    QMakeGlobals::parseProperties(output, *versionInfo);
+
     return true;
 }
 
-FileName BaseQtVersion::mkspecDirectoryFromVersionInfo(const QHash<QString, QString> &versionInfo)
+FileName BaseQtVersion::mkspecDirectoryFromVersionInfo(const QHash<ProKey, ProString> &versionInfo)
 {
     QString dataDir = qmakeProperty(versionInfo, "QT_HOST_DATA", PropertyVariantSrc);
     if (dataDir.isEmpty())
@@ -1484,7 +1488,7 @@ FileName BaseQtVersion::mkspecDirectoryFromVersionInfo(const QHash<QString, QStr
     return FileName::fromUserInput(dataDir + QLatin1String("/mkspecs"));
 }
 
-FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<QString, QString> &versionInfo)
+FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<ProKey, ProString> &versionInfo)
 {
     FileName baseMkspecDir = mkspecDirectoryFromVersionInfo(versionInfo);
     if (baseMkspecDir.isEmpty())
@@ -1513,10 +1517,11 @@ FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<QString, QString> &ver
                         if (temp.size() == 2) {
                             QString possibleFullPath = QString::fromLocal8Bit(temp.at(1).trimmed().constData());
                             if (possibleFullPath.contains(QLatin1Char('$'))) { // QTBUG-28792
-                                const QRegExp rex(QLatin1String("\\binclude\\(([^)]+)/qmake\\.conf\\)"));
-                                if (rex.indexIn(QString::fromLocal8Bit(f2.readAll())) != -1) {
+                                const QRegularExpression rex(QLatin1String("\\binclude\\(([^)]+)/qmake\\.conf\\)"));
+                                const QRegularExpressionMatch match = rex.match(QString::fromLocal8Bit(f2.readAll()));
+                                if (match.hasMatch()) {
                                     possibleFullPath = mkspecFullPath.toString() + QLatin1Char('/')
-                                            + rex.cap(1);
+                                            + match.captured(1);
                                 }
                             }
                             // We sometimes get a mix of different slash styles here...
@@ -1563,7 +1568,7 @@ FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<QString, QString> &ver
     return mkspecFullPath;
 }
 
-FileName BaseQtVersion::sourcePath(const QHash<QString, QString> &versionInfo)
+FileName BaseQtVersion::sourcePath(const QHash<ProKey, ProString> &versionInfo)
 {
     const QString qt5Source = qmakeProperty(versionInfo, "QT_INSTALL_PREFIX/src");
     if (!qt5Source.isEmpty())
@@ -1572,8 +1577,7 @@ FileName BaseQtVersion::sourcePath(const QHash<QString, QString> &versionInfo)
     const QString installData = qmakeProperty(versionInfo, "QT_INSTALL_PREFIX");
     QString sourcePath = installData;
     QFile qmakeCache(installData + QLatin1String("/.qmake.cache"));
-    if (qmakeCache.exists()) {
-        qmakeCache.open(QIODevice::ReadOnly | QIODevice::Text);
+    if (qmakeCache.exists() && qmakeCache.open(QIODevice::ReadOnly | QIODevice::Text)) {
         QTextStream stream(&qmakeCache);
         while (!stream.atEnd()) {
             QString line = stream.readLine().trimmed();
@@ -1601,7 +1605,7 @@ bool BaseQtVersion::isInSourceDirectory(const Utils::FileName &filePath)
     return filePath.isChildOf(dir);
 }
 
-bool BaseQtVersion::isSubProject(const Utils::FileName &filePath)
+bool BaseQtVersion::isSubProject(const Utils::FileName &filePath) const
 {
     const Utils::FileName &source = sourcePath();
     if (!source.isEmpty()) {
@@ -1690,11 +1694,13 @@ bool BaseQtVersion::isQtQuickCompilerSupported(QString *reason) const
     return true;
 }
 
-FileNameList BaseQtVersion::qtCorePaths(const QHash<QString,QString> &versionInfo, const QString &versionString)
+FileNameList BaseQtVersion::qtCorePaths() const
 {
+    const QString &versionString = qtVersionString();
+
     QStringList dirs;
-    dirs << qmakeProperty(versionInfo, "QT_INSTALL_LIBS")
-         << qmakeProperty(versionInfo, "QT_INSTALL_BINS");
+    dirs << qmakeProperty(versionInfo(), "QT_INSTALL_LIBS")
+         << qmakeProperty(versionInfo(), "QT_INSTALL_BINS");
 
     FileNameList staticLibs;
     FileNameList dynamicLibs;
@@ -1733,12 +1739,234 @@ FileNameList BaseQtVersion::qtCorePaths(const QHash<QString,QString> &versionInf
     return dynamicLibs;
 }
 
+static QByteArray scanQtBinaryForBuildString(const FileName &library)
+{
+    QFile lib(library.toString());
+    QByteArray buildString;
+
+    if (lib.open(QIODevice::ReadOnly)) {
+        const QByteArray startNeedle = "Qt ";
+        const QByteArray buildNeedle = " build; by ";
+        const size_t oneMiB = 1024 * 1024;
+        const size_t keepSpace = 4096;
+        const size_t bufferSize = oneMiB + keepSpace;
+        QByteArray buffer(bufferSize, Qt::Uninitialized);
+
+        char *const readStart = buffer.data() + keepSpace;
+        auto readStartIt = buffer.begin() + keepSpace;
+        const auto copyStartIt = readStartIt + (oneMiB - keepSpace);
+
+        while (!lib.atEnd()) {
+            const int read = lib.read(readStart, static_cast<int>(oneMiB));
+            const auto readEndIt = readStart + read;
+            auto currentIt = readStartIt;
+
+            forever {
+                const auto qtFoundIt = std::search(currentIt, readEndIt,
+                                                   startNeedle.begin(), startNeedle.end());
+                if (qtFoundIt == readEndIt)
+                    break;
+
+                currentIt = qtFoundIt + 1;
+
+                // Found "Qt ", now find the next '\0'.
+                const auto nullFoundIt = std::find(qtFoundIt, readEndIt, '\0');
+                if (nullFoundIt == readEndIt)
+                    break;
+
+                // String much too long?
+                const size_t len = std::distance(qtFoundIt, nullFoundIt);
+                if (len > keepSpace)
+                    continue;
+
+                // Does it contain " build; by "?
+                const auto buildByFoundIt = std::search(qtFoundIt, nullFoundIt,
+                                                        buildNeedle.begin(), buildNeedle.end());
+                if (buildByFoundIt == nullFoundIt)
+                    continue;
+
+                buildString = QByteArray(qtFoundIt, static_cast<int>(len));
+                break;
+            }
+
+            if (!buildString.isEmpty() || readEndIt != buffer.constEnd())
+                break;
+
+            std::move(copyStartIt, readEndIt, buffer.begin()); // Copy last section to front.
+        }
+    }
+    return buildString;
+}
+
+static QStringList extractFieldsFromBuildString(const QByteArray &buildString)
+{
+    if (buildString.isEmpty()
+            || buildString.count() > 4096)
+        return QStringList();
+
+    const QRegularExpression buildStringMatcher("^Qt "
+                                                "([\\d\\.a-zA-Z]*) " // Qt version
+                                                "\\("
+                                                "([\\w_-]+) "       // Abi information
+                                                "(shared|static) (?:\\(dynamic\\) )?"
+                                                "(debug|release)"
+                                                " build; by "
+                                                "(.*)"               // compiler with extra info
+                                                "\\)$");
+
+    QTC_ASSERT(buildStringMatcher.isValid(), qWarning() << buildStringMatcher.errorString());
+
+    const QRegularExpressionMatch match = buildStringMatcher.match(QString::fromUtf8(buildString));
+    if (!match.hasMatch())
+        return QStringList();
+
+    QStringList result;
+    result.append(match.captured(1)); // qtVersion
+
+    // Abi info string:
+    QStringList abiInfo = match.captured(2).split('-', QString::SkipEmptyParts);
+
+    result.append(abiInfo.takeFirst()); // cpu
+
+    const QString endian = abiInfo.takeFirst();
+    QTC_ASSERT(endian.endsWith("_endian"), return QStringList());
+    result.append(endian.left(endian.count() - 7)); // without the "_endian"
+
+    result.append(abiInfo.takeFirst()); // pointer
+
+    if (abiInfo.isEmpty()) {
+        // no extra info whatsoever:
+        result.append(""); // qreal is unset
+        result.append(""); // extra info is unset
+    } else {
+        const QString next = abiInfo.at(0);
+        if (next.startsWith("qreal_")) {
+            abiInfo.takeFirst();
+            result.append(next.mid(6)); // qreal: without the "qreal_" part;
+        } else {
+            result.append(""); // qreal is unset!
+        }
+
+        result.append(abiInfo.join('-')); // extra abi strings
+    }
+
+    result.append(match.captured(3)); // linkage
+    result.append(match.captured(4)); // buildType
+    result.append(match.captured(5)); // compiler
+
+    return result;
+}
+
+static Abi refineAbiFromBuildString(const QByteArray &buildString, const Abi &probableAbi)
+{
+    QStringList buildStringData = extractFieldsFromBuildString(buildString);
+    if (buildStringData.count() != 9)
+        return probableAbi;
+
+    const QString compiler = buildStringData.at(8);
+
+    Abi::Architecture arch = probableAbi.architecture();
+    Abi::OS os = probableAbi.os();
+    Abi::OSFlavor flavor = probableAbi.osFlavor();
+    Abi::BinaryFormat format = probableAbi.binaryFormat();
+    unsigned char wordWidth = probableAbi.wordWidth();
+
+    if (compiler.startsWith("GCC ") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMSysFlavor;
+    } else if (compiler.startsWith("MSVC 2005")  && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2005Flavor;
+    } else if (compiler.startsWith("MSVC 2008") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2008Flavor;
+    } else if (compiler.startsWith("MSVC 2010") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2010Flavor;
+    } else if (compiler.startsWith("MSVC 2012") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2012Flavor;
+    } else if (compiler.startsWith("MSVC 2015") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2015Flavor;
+    } else if (compiler.startsWith("MSVC 2017") && os == Abi::WindowsOS) {
+        flavor = Abi::WindowsMsvc2017Flavor;
+    }
+
+    return Abi(arch, os, flavor, format, wordWidth);
+}
+
+static Abi scanQtBinaryForBuildStringAndRefineAbi(const FileName &library,
+                                                   const Abi &probableAbi)
+{
+    static QHash<Utils::FileName, Abi> results;
+
+    if (!results.contains(library)) {
+        const QByteArray buildString = scanQtBinaryForBuildString(library);
+        results.insert(library, refineAbiFromBuildString(buildString, probableAbi));
+    }
+    return results.value(library);
+}
+
 QList<Abi> BaseQtVersion::qtAbisFromLibrary(const FileNameList &coreLibraries)
 {
     QList<Abi> res;
-    foreach (const FileName &library, coreLibraries)
-        foreach (const Abi &abi, Abi::abisOfBinary(library))
-            if (!res.contains(abi))
-                res.append(abi);
+    foreach (const FileName &library, coreLibraries) {
+        for (Abi abi : Abi::abisOfBinary(library)) {
+            Abi tmp = abi;
+            if (abi.osFlavor() == Abi::UnknownFlavor)
+                tmp = scanQtBinaryForBuildStringAndRefineAbi(library, abi);
+            if (!res.contains(tmp))
+                res.append(tmp);
+        }
+    }
     return res;
 }
+
+#if defined(WITH_TESTS)
+
+#include <QTest>
+
+#include "qtsupportplugin.h"
+
+void QtSupportPlugin::testQtBuildStringParsing_data()
+{
+    QTest::addColumn<QByteArray>("buildString");
+    QTest::addColumn<QString>("expected");
+
+    QTest::newRow("invalid build string")
+            << QByteArray("Qt with invalid buildstring") << QString();
+    QTest::newRow("empty build string")
+            << QByteArray("") << QString();
+    QTest::newRow("huge build string")
+            << QByteArray(8192, 'x') << QString();
+
+    QTest::newRow("valid build string")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64 shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;;;shared;release;GCC 6.2.1 20160830";
+
+    QTest::newRow("with qreal")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64-qreal___fp16 shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;__fp16;;shared;release;GCC 6.2.1 20160830";
+    QTest::newRow("with qreal and abi")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64-qreal___fp16-eabi shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;__fp16;eabi;shared;release;GCC 6.2.1 20160830";
+    QTest::newRow("with qreal, eabi and softfloat")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64-qreal___fp16-eabi-softfloat shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;__fp16;eabi-softfloat;shared;release;GCC 6.2.1 20160830";
+    QTest::newRow("with eabi")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64-eabi shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;;eabi;shared;release;GCC 6.2.1 20160830";
+    QTest::newRow("with eabi and softfloat")
+            << QByteArray("Qt 5.7.1 (x86_64-little_endian-lp64-eabi-softfloat shared (dynamic) release build; by GCC 6.2.1 20160830)")
+            << "5.7.1;x86_64;little;lp64;;eabi-softfloat;shared;release;GCC 6.2.1 20160830";
+}
+
+void QtSupportPlugin::testQtBuildStringParsing()
+{
+    QFETCH(QByteArray, buildString);
+    QFETCH(QString, expected);
+
+    QStringList expectedList;
+    if (!expected.isEmpty())
+        expectedList = expected.split(';');
+
+    QStringList actual = extractFieldsFromBuildString(buildString);
+    QCOMPARE(expectedList, actual);
+}
+
+#endif // WITH_TESTS
