@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,34 +9,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef TEXTEDITORCONSTANTS_H
-#define TEXTEDITORCONSTANTS_H
+#pragma once
 
 #include <QtGlobal>
 
 namespace TextEditor {
 
 // Text color and style categories
-enum TextStyle {
+enum TextStyle : quint8 {
     C_TEXT,
 
     C_LINK,
@@ -45,6 +39,8 @@ enum TextStyle {
     C_SEARCH_RESULT,
     C_SEARCH_SCOPE,
     C_PARENTHESES,
+    C_PARENTHESES_MISMATCH,
+    C_AUTOCOMPLETE,
     C_CURRENT_LINE,
     C_CURRENT_LINE_NUMBER,
     C_OCCURRENCES,
@@ -55,6 +51,7 @@ enum TextStyle {
     C_STRING,
     C_TYPE,
     C_LOCAL,
+    C_GLOBAL,
     C_FIELD,
     C_ENUMERATION,
     C_VIRTUAL_METHOD,
@@ -95,6 +92,16 @@ enum TextStyle {
     C_DIFF_DEST_LINE,
     C_DIFF_DEST_CHAR,
 
+    C_LOG_CHANGE_LINE,
+
+    C_WARNING,
+    C_WARNING_CONTEXT,
+    C_ERROR,
+    C_ERROR_CONTEXT,
+
+    C_DECLARATION,
+    C_OUTPUT_ARGUMENT,
+
     C_LAST_STYLE_SENTINEL
 };
 
@@ -103,6 +110,7 @@ namespace Constants {
 const char C_TEXTEDITOR[]          = "Text Editor";
 const char COMPLETE_THIS[]         = "TextEditor.CompleteThis";
 const char QUICKFIX_THIS[]         = "TextEditor.QuickFix";
+const char SHOWCONTEXTMENU[]       = "TextEditor.ShowContextMenu";
 const char CREATE_SCRATCH_BUFFER[] = "TextEditor.CreateScratchBuffer";
 const char VISUALIZE_WHITESPACE[]  = "TextEditor.VisualizeWhitespace";
 const char CLEAN_WHITESPACE[]      = "TextEditor.CleanWhitespace";
@@ -121,6 +129,7 @@ const char GOTO_BLOCK_END[]        = "TextEditor.GotoBlockEnd";
 const char GOTO_BLOCK_END_WITH_SELECTION[] = "TextEditor.GotoBlockEndWithSelection";
 const char SELECT_BLOCK_UP[]       = "TextEditor.SelectBlockUp";
 const char SELECT_BLOCK_DOWN[]     = "TextEditor.SelectBlockDown";
+const char SELECT_WORD_UNDER_CURSOR[]   = "TextEditor.SelectWordUnderCursor";
 const char VIEW_PAGE_UP[] = "TextEditor.viewPageUp";
 const char VIEW_PAGE_DOWN[] = "TextEditor.viewPageDown";
 const char VIEW_LINE_UP[] = "TextEditor.viewLineUp";
@@ -136,10 +145,14 @@ const char UPPERCASE_SELECTION[]   = "TextEditor.UppercaseSelection";
 const char LOWERCASE_SELECTION[]   = "TextEditor.LowercaseSelection";
 const char CUT_LINE[]              = "TextEditor.CutLine";
 const char COPY_LINE[]             = "TextEditor.CopyLine";
+const char DUPLICATE_SELECTION[]   = "TextEditor.DuplicateSelection";
+const char DUPLICATE_SELECTION_AND_COMMENT[] = "TextEditor.DuplicateSelectionAndComment";
 const char DELETE_LINE[]           = "TextEditor.DeleteLine";
 const char DELETE_END_OF_WORD[]    = "TextEditor.DeleteEndOfWord";
+const char DELETE_END_OF_LINE[]    = "TextEditor.DeleteEndOfLine";
 const char DELETE_END_OF_WORD_CAMEL_CASE[] = "TextEditor.DeleteEndOfWordCamelCase";
 const char DELETE_START_OF_WORD[]  = "TextEditor.DeleteStartOfWord";
+const char DELETE_START_OF_LINE[]  = "TextEditor.DeleteStartOfLine";
 const char DELETE_START_OF_WORD_CAMEL_CASE[] = "TextEditor.DeleteStartOfWordCamelCase";
 const char SELECT_ENCODING[]       = "TextEditor.SelectEncoding";
 const char REWRAP_PARAGRAPH[]      =  "TextEditor.RewrapParagraph";
@@ -175,6 +188,9 @@ const char FOLLOW_SYMBOL_UNDER_CURSOR_IN_NEXT_SPLIT[] = "TextEditor.FollowSymbol
 const char JUMP_TO_FILE_UNDER_CURSOR[] = "TextEditor.JumpToFileUnderCursor";
 const char JUMP_TO_FILE_UNDER_CURSOR_IN_NEXT_SPLIT[] = "TextEditor.JumpToFileUnderCursorInNextSplit";
 
+const char SCROLL_BAR_SEARCH_RESULT[] = "TextEditor.ScrollBarSearchResult";
+const char SCROLL_BAR_CURRENT_LINE[] = "TextEditor.ScrollBarCurrentLine";
+
 const char *nameForStyle(TextStyle style);
 TextStyle styleFromName(const char *name);
 
@@ -199,5 +215,3 @@ const unsigned COMPLETION_ASSIST_TOOLTIP_DELAY = 100;
 
 } // namespace Constants
 } // namespace TextEditor
-
-#endif // TEXTEDITORCONSTANTS_H
