@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -61,7 +56,7 @@ WinRtPackageDeploymentStep::WinRtPackageDeploymentStep(BuildStepList *bsl)
     m_args = defaultWinDeployQtArguments();
 }
 
-bool WinRtPackageDeploymentStep::init()
+bool WinRtPackageDeploymentStep::init(QList<const BuildStep *> &earlierSteps)
 {
     WinRtRunConfiguration *rc = qobject_cast<WinRtRunConfiguration *>(
                 target()->activeRunConfiguration());
@@ -108,7 +103,7 @@ bool WinRtPackageDeploymentStep::init()
     params->setArguments(args);
     params->setEnvironment(target()->activeBuildConfiguration()->environment());
 
-    return AbstractProcessStep::init();
+    return AbstractProcessStep::init(earlierSteps);
 }
 
 void WinRtPackageDeploymentStep::run(QFutureInterface<bool> &fi)
@@ -246,7 +241,7 @@ void WinRtPackageDeploymentStep::raiseError(const QString &errorMessage)
     ProjectExplorer::Task task = Task(Task::Error, errorMessage, Utils::FileName(), -1,
                                       ProjectExplorer::Constants::TASK_CATEGORY_DEPLOYMENT);
     emit addTask(task, 1);
-    emit addOutput(errorMessage, BuildStep::ErrorMessageOutput);
+    emit addOutput(errorMessage, BuildStep::OutputFormat::ErrorMessage);
 }
 
 void WinRtPackageDeploymentStep::raiseWarning(const QString &warningMessage)
@@ -254,7 +249,7 @@ void WinRtPackageDeploymentStep::raiseWarning(const QString &warningMessage)
     ProjectExplorer::Task task = Task(Task::Warning, warningMessage, Utils::FileName(), -1,
                                       ProjectExplorer::Constants::TASK_CATEGORY_DEPLOYMENT);
     emit addTask(task, 1);
-    emit addOutput(warningMessage, BuildStep::MessageOutput);
+    emit addOutput(warningMessage, BuildStep::OutputFormat::NormalMessage);
 }
 
 bool WinRtPackageDeploymentStep::fromMap(const QVariantMap &map)
